/*********************************************************************************************
 * File:    AbstractEntety.java
 * Project: Indoor maps, Masterthesis
 * Version: 0.5b
 * Author:  Jon Villy Meidell
 * Org:     NTNU
 * 
 * 
 * Data Structure
 * A Building
 */
package no.ntnu.map.indoormaps;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.IntBuffer;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.microedition.khronos.opengles.GL10;

import org.apache.http.entity.ContentProducer;
import org.json.JSONObject;

import no.ntnu.map.indoormaps.Storage.MysqlViaPHP;
import no.ntnu.map.indoormaps.Storage.SQLStorage.QueryBuilder;
import no.ntnu.map.indoormaps.Storage.SQLStorage.QueryBuilder.WrongTypeException;

import android.content.Context;
import android.graphics.Color;
import android.provider.BaseColumns;

// TODO: Auto-generated Javadoc
/**
 * An Abstrac Class defining default fields and functions requiered as datastructure and database entety.
 */
public abstract class AbstractEntety implements BaseColumns
{

	/** TAG used with {@link IndMap#log(short, String, String)}. */
	private static final String TAG="AbstractEntety";

	/**
	 * Interface for SQL functions needed by an {@link AbstractEntety}.
	 */
	public static interface SQLEntety
	{

		/** The Constant DROPTABLESQL. */
		public static final String DROPTABLESQL="DROP TABLE IF EXISTS ";

		/**
		 * Return the SQL String needed to create the {@link AbstractEntety AbstractEntety's} table.
		 *
		 * @return SQL String
		 * @throws WrongTypeException the wrong type exception
		 */
		public String createTable() throws WrongTypeException;

		/**
		 * Return the SQL String needed to drop the {@link AbstractEntety AbstractEntety's} table.
		 *
		 * @return SQL String
		 * @throws WrongTypeException the wrong type exception
		 */
		public String dropTable() throws WrongTypeException;

		/**
		 * Return the SQL String to get all lines from the {@link AbstractEntety AbstractEntety's} table.
		 *
		 * @return SQL String
		 * @throws WrongTypeException the wrong type exception
		 */
		public String getAll() throws WrongTypeException;

		/**
		 * Return the SQL String to get all lines given parent(foreign key) from the {@link AbstractEntety AbstractEntety's} table.
		 *
		 * @param parent The parent
		 * @return SQL String
		 * @throws WrongTypeException the wrong type exception
		 */
		public String getByParent(AbstractEntety parent) throws WrongTypeException;

		/**
		 * Return the SQL String to get all lines given parent(foreign key)'s id from the {@link AbstractEntety AbstractEntety's} table.
		 *
		 * @param parent_id the parent_id
		 * @return SQL String
		 * @throws WrongTypeException the wrong type exception
		 */
		public String getByParent(int parent_id) throws WrongTypeException;

		/**
		 * Return the SQL String to get one lines given id from the {@link AbstractEntety AbstractEntety's} table.
		 *
		 * @param id {@link BaseColumns#_ID _ID} to given row
		 * @return SQL String
		 * @throws WrongTypeException the wrong type exception
		 */
		public String getOne(int id) throws WrongTypeException;
	}

	/**
	 * Extended {@link SQLEntety} to get SQL Query String to fetch shapes where no {@link AbstractEntety#vFloor Floor vertex buffer} or.
	 *
	 * {@link AbstractEntety#vFloor Wall vertex buffer} is saved in the external database, so can generate the buffers
	 * on loadtime. (With other words: functions to fall back from real 3D, to semi 3D from 2D
	 * @see SQLEntety
	 * @see Shape
	 */
	public static interface ShapeSQLEntety extends SQLEntety
	{

		/**
		 * Return the SQL String needed to fetch all shapes and its vertexes from the external database.
		 *
		 * @param obj The {@link AbstractEntety} who needs its shapes
		 * @param match What kind of {@link Shape} that is requested.
		 * @return SQL String
		 * @throws WrongTypeException the wrong type exception
		 * @see Shape
		 */
		public String getShapes(AbstractEntety obj,short match) throws WrongTypeException;
	}

	/**
	 * An object where to store fetched shapes from external database when {@link AbstractEntety#vFloor Floor vertex buffer} is missing.
	 * <br><br>
	 * 
	 * Uses the table {@link Shape#TABLE_NAME} to link with vertexes.<br>
	 * Fields:<ul>
	 * <li>{@link Shape#ID}</li>
	 * <li>{@link Shape#TYPE}</li>
	 * <li>{@link Shape#PRI}</li>
	 * </ul>
	 * <br><br>
	 * Wall types: as of <ul>
	 * <li> {@link Shape#TYPE_WALL} with identifier {@link Shape#WALL}</li>
	 * <li> {@link Shape#TYPE_WIN} with identifier {@link Shape#WINDOW}</li>
	 * <li> {@link Shape#TYPE_DOOR} with identifier {@link Shape#DOOR}</li>
	 * <li> {@link Shape#TYPE_BASE} with identifier {@link Shape#BASE}</li>
	 * </ul>
	 * 
	 * @see ShapeSQLEntety
	 */
	public static class Shape
	{

		/** Its a Wall Shape. */
		public static final short WALL=1;

		/** Its a Wall Shape. */
		public static final String TYPE_WALL="wall";

		/** Its a Window Shape. */
		public static final short WINDOW=2;

		/** Its a Window Shape. */
		public static final String TYPE_WIN="window";

		/** Its a Door Shape. */
		public static final short DOOR=3;

		/** Its a Door Shape. */
		public static final String TYPE_DOOR="door";

		/** Its a Base Shape. */
		public static final short BASE=4;

		/** Its a Base Shape. */
		public static final String TYPE_BASE="base";

		/** The Database table name. Used in {@link Provider} and calls to it, {@link QueryBuilder}, ect*/
		public static final String TABLE_NAME="shape";

		/** The parent key as foreign key. */
		public static final String PARENT="shape_id";


		/** Database field id. */
		public static final String ID="id";

		/** Database field table.id for use in advanced queries. */
		public static final String _ID=TABLE_NAME+"."+ID;

		/** Database field type field. Holds the values {@link #TYPE_WALL}, {@link #TYPE_WIN}, {@link #TYPE_DOOR} or {@link #TYPE_BASE} */
		public static final String TYPE="type";

		/** Database field pri, for sorting vertex drawing order. */
		public static final String PRI="pri";


		/** Predefined wall height when making semi 3D from 2D. */
		public static final short WALL_HEIGHT=90;

		/** Predefined window height when making semi 3D from 2D. */
		public static final short WINDOW_HEIGHT=30;

		/** Predefined height from floor to window when making semi 3D from 2D. */
		public static final short HEIGHT2WINDOW=40;

		/** Predefined height from wall to ceiling when making semi 3D from 2D. */
		public static final short HEIGHTFROMWINDOW=WINDOW_HEIGHT+HEIGHT2WINDOW;

		/** Predefined door height when making semi 3D from 2D. */
		public static final short DOOR_HEIGHT=70;

		/** Predefined height from door to ceiling when making semi 3D from 2D. */
		public static final short HEIGHTFROMDOOR=WALL_HEIGHT-DOOR_HEIGHT;
		
		public static final short SPACE_BETWEEN_FLOORS=10;

		/** The _id corresponding to {@link #_ID}. */
		private int _id;

		/** Array to keep loaded vertexes. */
		private ArrayList<Point> vertexes=new ArrayList<Point>();

		/**
		 * Instantiates a new shape, and sets its id.
		 *
		 * @param id The shapes' id.
		 */
		public Shape(int id){_id=id;}

		/**
		 * Gets the Shapes' id.
		 *
		 * @return id
		 */
		public int getI(){return _id;}

		/**
		 * Adds a vertex.
		 *
		 * @param x x-coordinate
		 * @param y y-coordinate
		 * @param z z-coordinate
		 */
		public void addVertex(float x, float y, float z){vertexes.add(new Point(x,y,z));}

		/**
		 * Adds a vertex by {@link Point}.
		 *
		 * @param p A {@link Point}
		 */
		public void addVertex(Point p){vertexes.add(p);}

		/**
		 * Adds a vertexes as {@link Point points}.
		 *
		 * @param vertexes {@link Point Points}
		 */
		public void addVertexes(Collection<Point> vertexes){this.vertexes.addAll(vertexes);}

		/**
		 * Gets the number of vertexes.
		 *
		 * @return the nuber of vertexes.
		 */
		public int getSize(){return vertexes.size();}

		/**
		 * Gets one vertex as {@link Point}.
		 *
		 * @param i The index of the vertex
		 * @return A vertex as {@link Point}
		 */
		public Point getVertex(int i){return vertexes.get(i);}

		/**
		 * Returns the iterator of the buildin vertex array.
		 *
		 * @return A {@link Iterator} of {@link Point Points}
		 */
		public Iterator<Point> getIterator(){return vertexes.iterator();}

		/**
		 * To check output from database to get the numerical representation
		 * 
		 * Wall types: as of <ul>
		 * <li> {@link Shape#TYPE_WALL} with identifier {@link Shape#WALL}</li>
		 * <li> {@link Shape#TYPE_WIN} with identifier {@link Shape#WINDOW}</li>
		 * <li> {@link Shape#TYPE_DOOR} with identifier {@link Shape#DOOR}</li>
		 * <li> {@link Shape#TYPE_BASE} with identifier {@link Shape#BASE}</li>
		 * </ul>.
		 *
		 * @param type The type as string
		 * @return The type as short
		 */
		public static short matchType(String type)
		{
			if(type.equals(TYPE_WALL))return WALL;
			if(type.equals(TYPE_WIN))return WINDOW;
			if(type.equals(TYPE_DOOR))return DOOR;
			if(type.equals(TYPE_BASE))return BASE;
			return -1;
		}

		/**
		 * Extends a query being build by a {@link QueryBuilder} to get SQL String to get required shapes for a {@link ShapeSQLEntety}.
		 *
		 * @param qb The {@link QueryBuilder}
		 * @param joinTab The {@link ShapeSQLEntety shape entety's} table name.
		 * @throws WrongTypeException if something wrong calling the {@link QueryBuilder}
		 * @see Building#TABLE_NAME
		 * @see Floor#TABLE_NAME
		 * @see Room#TABLE_NAME
		 */
		public static void join2ParentAndPoint(QueryBuilder qb,String joinTab) throws WrongTypeException
		{
			//Needs like qb.addTable(Building.TABLE_NAME+Shape.TABLE_NAME); in calling method to complete query
			//To use in getShapes(AbstractEntety obj,short match) functions, like Building.Entety (SQLEntety)
			qb.addJoinCond(joinTab+"."+Shape.PARENT, QueryBuilder.EQUALS, Shape._ID);
			qb.addTable(Shape.TABLE_NAME);
			String table=Shape.TABLE_NAME+Point.TABLE_NAME; //shapevertex
			qb.addJoinCond(Shape._ID, QueryBuilder.EQUALS, table+"."+Shape.PARENT);
			qb.addTable(table);
			qb.addJoinCond(Point._ID, QueryBuilder.EQUALS, table+"."+Point.PARENT);
			qb.addTable(Point.TABLE_NAME);
			qb.addFields(new String[]{Shape._ID,Shape.TYPE,Point.X,Point.Y});
			qb.addOrdering(Shape._ID); //Get the shapes grouped up
			qb.addOrdering(Shape.PRI); //Get drawing priority for vertexes
		}
	}

	/** The  {@link SQLEntety}. @uml.property  name="entety" @uml.associationEnd */
	private SQLEntety entety;

	/**
	 * Get the  {@link SQLEntety}.
	 *
	 * @return  the entety
	 * @uml.property  name="entety"
	 */
	public SQLEntety getEntety(){return entety;}

	/**
	 * Set the  {@link SQLEntety}.
	 *
	 * @param entety  the new entety
	 * @uml.property  name="entety"
	 */
	public void setEntety(SQLEntety entety){this.entety = entety;}

	/** The id, not like the unique numerical  {@link BaseColumns#_ID}  -  {@link #_id}. @uml.property  name="id" */
	protected String id;

	/**
	 * The name.
	 * @uml.property  name="name"
	 */
	protected String name="<UNKNOWN>";

	/**
	 * The unique id.
	 * @uml.property  name="_id"
	 */
	protected int _id;

	/**
	 * The v wall buffer. For the  {@link GL10#glVertexPointer(int,int,int,java.nio.Buffer)} 
	 * @uml.property  name="vWallBuffer"
	 */
	IntBuffer vWallBuffer;

	/**
	 * The v floor buffer. For the  {@link GL10#glVertexPointer(int,int,int,java.nio.Buffer)} 
	 * @uml.property  name="vFloorBuffer"
	 */
	IntBuffer vFloorBuffer;

	/** An int array holding vertexes before generated to  {@link #vFloorBuffer}. @uml.property  name="vWall" multiplicity="(0 -1)" dimension="1" */
	int[] vWall=new int[0];

	/** An int array holding vertexes before generated to  {@link #vWallBuffer}. @uml.property  name="vFloor" multiplicity="(0 -1)" dimension="1" */
	int[] vFloor=new int[0];

	/** The number of wall vertexes in the buffer, required by  {@link GL10#glDrawArrays(int,int,int)}. @uml.property  name="num_vWall" */
	int num_vWall=0;

	/** The number of floor vertexes in the buffer, required by  {@link GL10#glDrawArrays(int,int,int)}. @uml.property  name="num_vFloor" */
	int num_vFloor=0;

	/**
	 * Return the wall buffer required by {@link GL10#glVertexPointer(int, int, int, java.nio.Buffer)}
	 *
	 * @return the wall buffer
	 */
	public IntBuffer getWallBuffer(){return vWallBuffer;}

	/**
	 * Return the floor buffer required by {@link GL10#glVertexPointer(int, int, int, java.nio.Buffer)}
	 *
	 * @return the floor buffer
	 */
	public IntBuffer getFloorBuffer(){return vFloorBuffer;}

	/**
	 * Gets the number of wall vertexes.
	 *
	 * @return number of vertexes
	 */
	public int getNumWall(){return num_vWall;}

	/**
	 * Gets the number of floor vertexes.
	 *
	 * @return number of vertexes
	 */
	public int getNumFloor(){return num_vFloor;}

	/** Default field for those extending {@link AbstractEntety}. */
	public static final String ID = "id";

	/** Default field for those extending {@link AbstractEntety}. */
	public static final String NAME = "name";

	/** Default field for those extending {@link AbstractEntety}. */
	public static final String WALL_BUF="wallbuf";

	/** Default field for those extending {@link AbstractEntety}. */
	public static final String FLOOR_BUF="floorbuf";

	/**
	 * Gets textual id.
	 * @return  the id
	 * @see  #getI()
	 * @uml.property  name="id"
	 */
	public String getId(){return id;}

	/**
	 * Gets numerical id.
	 *
	 * @return the id
	 * 
	 * @see #getId()
	 */
	public int getI(){return _id;}

	/**
	 * Gets the name.
	 * @return  the name
	 * @uml.property  name="name"
	 */
	public String getName(){return name;}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object object)
	{
		if (object instanceof AbstractEntety)return equals((AbstractEntety) object);
		return false;
	}

	/**
	 * Extended equals support. Two {@link AbstractEntety} is the same type, and have the same {@link #_id}
	 *
	 * @param object The {@link AbstractEntety} to compare to.
	 * @return true, if equal
	 */
	public boolean equals(AbstractEntety object)
	{
		if(object==null) return false;
		if(!this.getClass().equals(object.getClass()))return false;
		return ((object._id==this._id)&&(object.getClass()==this.getClass()));
	}

	/**
	 * Searches an array of {@link AbstractEntety}'s that is sorted ascending by {@link #_id}, using binary search.
	 *
	 * @param elms The elements
	 * @param id The {@link #_id} of the {@link AbstractEntety}
	 * @return -1 if not found, the index of the found element otherwise.
	 */
	public static int getIndexById(AbstractEntety[] elms,int id){return searchById(elms,0,elms.length,id);}

	/**
	 * Searches an array of {@link AbstractEntety}'s that is sorted ascending by {@link #_id}, using binary search.
	 *
	 * @param elms The elements
	 * @param id The {@link #_id} of the {@link AbstractEntety} to search for
	 * @return null if not found, the found element otherwise
	 */
	public static AbstractEntety searchById(AbstractEntety[] elms,int id)
	{
		int i=searchById(elms,0,elms.length,id);
		if(i<0)return null;
		else return elms[i];
	}	

	/**
	 * The implemented binary-serach.
	 *
	 * @param elms The elements to search
	 * @param min The low, minimum index for the search interval of the elements
	 * @param max The high, maximum index for the search interval of the elements
	 * @param id The id to search for.
	 * @return -1 if no element with the id is found, otherwise the index where found.
	 */
	private static int searchById(AbstractEntety[] elms,int min,int max,int id)
	{
		//Recursive binary search
		//		IndMap.log(IndMap.DEBUG, TAG+"-binsearch", "Got "+elms.length+" elems , min: "+min+" max: "+max);
		int a=max-min;
		if(a < 1)return -1;
		int i = min + (a%2 == 1 ? a-1 : a) / 2;
		//		IndMap.log(IndMap.DEBUG, TAG+"-binsearch", "Center: "+i+ " "+id+" = "+elms[i].getI()+" ??");
		if(elms[i].getI()==id)return i;
		else if(a<2) return -1;
		else if(elms[i].getI()<id)return searchById(elms, i + 1, max, id);
		else return searchById(elms, min, i, id);
	}

	/**
	 * Checks if the array sorted ascending by {@link #_id} contains an given element.
	 * Uses {@link #searchById(AbstractEntety[], int)}
	 *
	 * @param elms The elements to search
	 * @param id The {@link #_id} of the {@link AbstractEntety} to search for
	 * @return Returns true, if found. False if not.
	 */
	public static boolean containsById(AbstractEntety[] elms,int id){return(searchById(elms, id)!=null);}

	/**
	 * Default draw method for all {@link AbstractEntety}'s.
	 *
	 * @param gl The OpenGL reference
	 * @param drawMode GL drawmode as of {@link GL10#GL_LINES}, {@link GL10#GL_LINE_STRIP}, {@link GL10#GL_TRIANGLES}, {@link GL10#GL_TRIANGLE_STRIP} or {@link GL10#GL_TRIANGLE_FAN}
	 * @param buffer The vertexbuffer in {@link GL10#GL_FIXED} format.
	 * @param num_vertexes Number of vertexes in the buffer
	 * @param color A color int, handeled by {@link Color} to extract red, green, blue and alpha.
	 */
	protected static void draw(GL10 gl,int drawMode,IntBuffer buffer,int num_vertexes,int color)
	{
		try
		{
			buffer.position(0);
			//			nbuffer.position(0);
			//gl.glColor4x(Color.red(color), Color.green(color), Color.blue(color), Color.alpha(color));
			gl.glColor4f (Color.red(color)/(float) 0xFF, Color.green(color)/(float) 0xFF, Color.blue(color)/(float) 0xFF, Color.alpha(color)/(float) 0xFF);
			//gl.glColor4f(1.0f, 0.3f, 0.0f, .5f);
			gl.glEnableClientState(GL10.GL_VERTEX_ARRAY);
			//gl.glNormalPointer(GL10.GL_SHORT, 0, nbuffer);
			//gl.glNormal3x(0, 0, -1);
			gl.glVertexPointer(3, GL10.GL_FIXED, 0, buffer);
			gl.glDrawArrays(drawMode, 0, num_vertexes);    
			gl.glDisableClientState(GL10.GL_VERTEX_ARRAY);
		}
		catch (Exception e) {
			IndMap.log(IndMap.ERROR, TAG, "Drawing "+e.toString());
		}
	}

	/**
	 * Draw the Object. May use {@link #draw(GL10, int, IntBuffer, int, int)} in its implementation.
	 *
	 * @param gl The OpenGL reference
	 * @param world The {@link World} reference, to use different colors according to selection in {@link World the world}.
	 * 
	 * @see #draw(GL10, int, IntBuffer, int, int)
	 */
	public abstract void draw(GL10 gl,World world);

	/**
	 * Convert float to fixed.
	 *
	 * @param x A float
	 * @return as fixed
	 */
	static public int toFixed(float x) {
		return (int)(x * 65536.0f);
	}

	/**
	 * Convert fixed tofloat.
	 *
	 * @param x The fixed
	 * @return A float
	 */
	static public float toFloat(int x) {
		return x/65536.0f;
	}

	/**
	 * Converte the {@link #vFloor} and {@link #vWall} into corresponding buffers {@link #vFloorBuffer} and {@link #vWallBuffer}
	 * And reset the int[] arrays.
	 */
	public void generateBuffers()
	{
		try{
			num_vFloor=vFloor.length/3;
			ByteBuffer bb = ByteBuffer.allocateDirect(num_vFloor*4*3);
			bb.order(ByteOrder.nativeOrder());
			vFloorBuffer = bb.asIntBuffer();
			vFloorBuffer.put(vFloor);

			num_vWall=vWall.length/3;
			bb = ByteBuffer.allocateDirect(num_vWall*4*3);
			bb.order(ByteOrder.nativeOrder());
			vWallBuffer = bb.asIntBuffer();
			vWallBuffer.put(vWall);

			vFloor=new int[0];
			vWall=new int[0];
			//IndMap.log(IndMap.DEBUG, TAG, "Generate buffers numFloor: "+num_vFloor+"  numWall: "+num_vWall);

		}catch (Exception e) {
			IndMap.log(IndMap.ERROR, TAG, "Generate buffers "+e.toString());
		}
	}

	/**
	 * Takes 2D vertexes and combines them with coordenates for z-axis to return two triangles as of fixed int vertexes
	 * <br><br>
	 * 
	 * Draws from right to left, surface facing. So {@link Point} A is to the right of {@link Point} B.
	 *
	 * @param a Point A
	 * @param b Point B
	 * @param zlow Low z-coord
	 * @param zhigh Heigh z-coord
	 * @return Fixed ints as {@link Integer} {@link ArrayList}
	 * 
	 * @see #toFixed(float)
	 */
	protected static ArrayList<Integer> from2Dto3D(Point a, Point b,float zlow,float zhigh)
	{
		ArrayList<Integer> p=new ArrayList<Integer>();
		//Tri 1
		p.add(new Integer(toFixed(a.x)));
		p.add(new Integer(toFixed(a.y)));
		p.add(new Integer(toFixed(zlow)));

		p.add(new Integer(toFixed(a.x)));
		p.add(new Integer(toFixed(a.y)));
		p.add(new Integer(toFixed(zhigh)));

		p.add(new Integer(toFixed(b.x)));
		p.add(new Integer(toFixed(b.y)));
		p.add(new Integer(toFixed(zhigh)));

		//Tri 2
		p.add(new Integer(toFixed(a.x)));
		p.add(new Integer(toFixed(a.y)));
		p.add(new Integer(toFixed(zlow)));

		p.add(new Integer(toFixed(b.x)));
		p.add(new Integer(toFixed(b.y)));
		p.add(new Integer(toFixed(zhigh)));

		p.add(new Integer(toFixed(b.x)));
		p.add(new Integer(toFixed(b.y)));
		p.add(new Integer(toFixed(zlow)));
		return p;
	}

	/**
	 * Generate a lot of triangle that covers a flat surface using the input vertexes as outer. Somehow adding polygon support. 
	 * <br>
	 * Only uses the x and y coordinates. 
	 * 
	 * @param vertexes The vertexes giving the corders of the surface
	 * @return {@link Point}'s giving tree and tree vertexes of triangles covering giving surface  
	 */
	protected ArrayList<Point> makeSurfaceTriangles(ArrayList<Point> vertexes)
	{
		boolean debug=false;

		ArrayList<Point> ret=new ArrayList<Point>(),retloc;
		try{
			//			ArrayList<Point> all=new ArrayList<Point>(vertexes);
			//for(i=0;i<vertexes.size();i++)done[i]=false;
			for(int i=0;i<vertexes.size()-2;i++)
			{
				//todo=new ArrayList<Point>(all);
				Point b=vertexes.get(i),a=vertexes.get(i+1),c=vertexes.get(i+2);

				ArrayList<Point> remain=new ArrayList<Point>(vertexes);
				remain.remove(c);
				remain.remove(a);
				remain.remove(b);
				if(debug)IndMap.log(IndMap.DEBUG, "makeSurface", "MOVE_FWD "+i+"***************************************************");
				if(debug)IndMap.log(IndMap.DEBUG, "makeSurface", "MOVE_FWD "+i+" - "+(i+2));

				//Check if triangle is convex, and no other random vertex is inside it.
				//If convex the vertex is done, also if sub-triangles made.
				retloc=makeTriang(remain,vertexes, b, a, c,debug);
				ret.addAll(retloc);

				//Check visited vertexes for new triangle combinations
				for(int j=i;j>2;j--)
				{
					b=vertexes.get(j-2);
					a=vertexes.get(j-1);
					c=vertexes.get(j);

					remain=new ArrayList<Point>(vertexes.subList(j+1, vertexes.size()));
					remain.remove(c);
					remain.remove(a);
					remain.remove(b);

					if(debug)IndMap.log(IndMap.DEBUG, "makeSurface", " - MOVE_BWD "+(j-2)+"********************");
					if(debug)IndMap.log(IndMap.DEBUG, "makeSurface", " - MOVE_BWD "+(j-2)+" - "+j);
					retloc=new ArrayList<Point>();
					retloc.addAll(makeTriang(remain,vertexes, b, a, c,debug));

					if(retloc.size()>0)
					{
						j--;
						i--;
						ret.addAll(retloc);
						if(debug)IndMap.log(IndMap.DEBUG, "makeSurface", " - MOVE_BWD Correcting index");	 
					}
				}
			}
		}catch (Exception e) {
			IndMap.log(IndMap.ERROR, TAG, "makeSurfaceTriangles "+e.toString());
		}
		return ret;
	}

	/**
	 * Investigates tree vertexes to find the angle between the to vectors. 
	 * <ul>
	 * <li>If concave, then returning empty list.</li>
	 * <li>If convext, then it checks wherever there is a vertex inside the triangle.
	 *     <ul>
	 *     <li> If there is, make subtriangles to avoid drawing outside the shape</li>
	 *     <li> Else returnung empty list signaling {@link #makeSurfaceTriangles(ArrayList)} to make a triangle between given {@link Point} a, b and c.
	 *     </ul></li>
	 * </ul>
	 *
	 * @param all All vertexes, so the method can check if a point is inside the tringale that {@link #makeSurfaceTriangles(ArrayList)} wants to draw.
	 * @param vertexes The remaining vertexes which is not done. Vertexes that gets done, is removed.
	 * @param b Point b, giving AB-vector
	 * @param a Point a, the center and tip of the angle
	 * @param c Point c, giving AC-vector
	 * @param debug The debugflag from {@link #makeSurfaceTriangles(ArrayList)}
	 * @return New triangles added if found vertex inside investigated {@link Point} a,b and c.
	 */
	private ArrayList<Point> makeTriang(ArrayList<Point> all,ArrayList<Point> vertexes,Point b, Point a, Point c,boolean debug)
	{
		ArrayList<Point> ret=new ArrayList<Point>();
		try{
			if(debug)IndMap.log(IndMap.DEBUG, "makeSurface", "**Got B"+b.toString()+ " A"+a.toString()+" C"+c.toString());

			double 	abdx=b.x-a.x,abdy=b.y-a.y, //v1
			acdx=c.x-a.x,acdy=c.y-a.y; //v0

			double sc=(acdx*(0-abdy)+acdy*abdx);
			if(debug)IndMap.log(IndMap.DEBUG, "makeSurface", "  - AB:["+abdx+","+abdy+"] AC:["+acdx+","+acdy+"]\nScalar: "+sc);
			if(sc>0) //If triangle is convex
			{
				if(debug)IndMap.log(IndMap.DEBUG, "makeSurface", "  - Convex!");
				boolean done=true;
				if(all.size()>0)for(int i=0;i<all.size();i++)
				{
					double apdx = (all.get(i).x - a.x) ,apdy = (all.get(i).y - a.y) ; //v2
					double scAC  = acdx*acdx+acdy*acdy; //dot00
					double scBAC = abdx*acdx+abdy*acdy; //dot10
					double scAB  = abdx*abdx+abdy*abdy; //dot11
					double scCAP = acdx*apdx+acdy*apdy; //dot02
					double scBAP = abdx*apdx+abdy*apdy; //dot12


					double div= 1.0 / (scAC*scAB - scBAC*scBAC); 
					double u=  (scAB*scCAP - scBAC*scBAP) * div;
					double v=  (scAC*scBAP - scBAC*scCAP) * div;
					if(u>0&&v>0&&(u+v)<1) //Check if exist some other vertex inside triangle, if so make sub-triangles including inside vertex
					{
						if(debug)IndMap.log(IndMap.DEBUG, "makeSurface", "  - Found vertex inside: "+all.get(i));
						if(!vertexes.contains(all.get(i)))
						{ //If a vertex who is done is inside the triangle we want to draw, skip and draw no triangle.
							if(debug)IndMap.log(IndMap.DEBUG, "makeSurface", "  - This is done, so continuing");
							done=false;
							continue;
						}
						//If a vertex is found inside the triangle we want to make, make sub-triangles. Vertex who is done, is already skipped
						ArrayList<Point> remain=new ArrayList<Point>(all);
						remain.remove(all.get(i));
						if(debug)IndMap.log(IndMap.DEBUG, "makeSurface", "  - SUB inside triangles: -------------");
						ret.addAll(makeTriang(remain,vertexes,b,a,all.get(i),debug)); //Substriangles left side
						if(i==0) ret.addAll(makeTriang(remain,vertexes,a,c,all.get(i),debug));//Substriangles right side
						else     ret.addAll(makeTriang(remain,vertexes,all.get(i),a,c,debug));//Substriangles right side
						if(debug)IndMap.log(IndMap.DEBUG, "makeSurface", "  -------------------------------------");
						break;
					}
				}
				if(ret.size()==0&&done)
				{
					if(debug)IndMap.log(IndMap.DEBUG, "makeSurface", "  -Found none so adding triangle");
					ret.add(b);
					ret.add(a);
					ret.add(c);
					vertexes.remove(a); //a is done when this
					if(debug)IndMap.log(IndMap.DEBUG, "makeSurface", "DONE!: "+a.toString());
				}
			}
		}catch (Exception e) {
			IndMap.log(IndMap.ERROR, TAG, "makeSurfaceTriangles "+e.toString());
		}
		return ret;
	}

	/**
	 * Store the element to the internal database.
	 *
	 * @param c The application context, to acces the {@link ContentProducer}.
	 */
	abstract void toInternal(Context c, int i);

	/**
	 * Fetch wallshapes from database and populates {@link #vWall}. 2D becomes semi 3D
	 *
	 * @param s The source to call new queries and get result from database.
	 * @param basehigh The zero-height for the floor.
	 * @throws Exception If somethings fuckes up!
	 */
	protected void wallFromDatabase(MysqlViaPHP s,int basehigh) throws Exception
	{
		//ArrayList<Shape> w=new ArrayList<Shape>();
		ArrayList<Integer> integ=new ArrayList<Integer>();
		if(s.next())
		{
			JSONObject json,lastjson=s.getJSONObject();
			int id,lastid=lastjson.getInt(Shape.ID);
			String type=null;

			while(s.next())
			{
				json=s.getJSONObject();
				id=json.getInt(Shape.ID);
				type=lastjson.getString(Shape.TYPE);
				if(lastid==id)
				{
					Point a=new Point(lastjson.getDouble(Point.X),lastjson.getDouble(Point.Y), 0.0);
					Point b=new Point(json.getDouble(Point.X),json.getDouble(Point.Y), 0.0);
					switch(Shape.matchType(type))
					{
						case Shape.WALL:
							integ.addAll(from2Dto3D(
									a, 
									b,
									basehigh, basehigh+Shape.WALL_HEIGHT));
							break;
						case Shape.WINDOW:
							integ.addAll(from2Dto3D(
									a,
									b,
									basehigh, basehigh+Shape.HEIGHT2WINDOW));
							integ.addAll(from2Dto3D(
									a,
									b,
									basehigh+Shape.HEIGHTFROMWINDOW, basehigh+Shape.WALL_HEIGHT)); 
							break;
						case Shape.DOOR:
							integ.addAll(from2Dto3D(
									a,
									b,
									basehigh+Shape.DOOR_HEIGHT, basehigh+Shape.WALL_HEIGHT));
							break;
						default:
					}
				}
				lastid=id;
				lastjson=json;
			}
		}
		vWall=new int[integ.size()];
		for(int i=0;i<integ.size();i+=3)
		{
			// P(X,Y,Z) OpenGl coords
			// P'(x',y',z') Database coords
			//X = y' Y = z' Z = -x'
			vWall[i]=integ.get(i+1);   //X = y'
			vWall[i+1]=integ.get(i+2); //Y = z'
			vWall[i+2]=-integ.get(i); //Z = -x'
		}
	}

	/** The max length. */
	private int maxLength=Integer.MIN_VALUE;
	
	/** The max width. */
	private int maxWidth=Integer.MIN_VALUE;
	
	/**
	 * Gets the max length.
	 *
	 * @return the max length
	 */
	public int getMaxLength(){return maxLength;}
	
	/**
	 * Gets the max width.
	 *
	 * @return the max width
	 */
	public int getMaxWidth(){return maxWidth;}

	/**
	 * Generate the floor by floor shapes from the database and populates the {@link #vFloor}.
	 *
	 * @param s The source to call new queries and get result from database.
	 * @param basehigh The zero-height for the floor.
	 * @throws Exception If somethings fuckes up!
	 */
	protected void floorFromDatabase(MysqlViaPHP s,int basehigh) throws Exception
	{
		maxLength=Integer.MIN_VALUE;
		maxWidth=Integer.MIN_VALUE;
		ArrayList<Point> vertexes=new ArrayList<Point>();
		int id=-1;
		while(s.next())
		{
			JSONObject json=s.getJSONObject();
			if(id<0) id=json.getInt(Shape.ID);
			else if(json.getInt(Shape.ID)!=id)break;
			double x=json.getDouble(Point.X), y=json.getDouble(Point.Y);
			if(x>maxLength)maxLength=(int) x + 1;
			if(y>maxWidth)maxWidth=(int) y + 1;
			vertexes.add(new Point(x, y, basehigh));
		}

		Iterator<Point> i=makeSurfaceTriangles(vertexes).iterator();
		ArrayList<Integer> integ=new ArrayList<Integer>();
		while(i.hasNext())
		{
			Point p=i.next();
			integ.add(new Integer(toFixed(p.y)));  //x = y'
			integ.add(new Integer(toFixed(p.z)));  //y = z'
			integ.add(new Integer(toFixed(-p.x))); //Z = -x'
		}
		vFloor=new int[integ.size()];
		for(int j=0;j<integ.size();j++)vFloor[j]=integ.get(j);
	}

	/**
	 * Convert a byte array to an int array.
	 *
	 * @param src byte array
	 * @return int array
	 */
	protected static int[] byte2Int(byte[] src)
	{
		int[] ret=new int[src.length >>> 2];
		int j=0;
		for(int i=0;i<ret.length;i++)
		{
			int x=0;
			x+=(src[j++] & 0xff) << 0;
			x+=(src[j++] & 0xff) << 8;
			x+=(src[j++] & 0xff) << 16;
			x+=(src[j++] & 0xff) << 24;
			ret[i]=x;
		}
		return ret;

		/*
		ByteBuffer bb=ByteBuffer.allocateDirect(bytes.length);
		return bb.order(ByteOrder.nativeOrder()).put(bytes).asIntBuffer();
		 */
	}

	/**
	 * Convert an int array to a byte array.
	 *
	 * @param src int array
	 * @return byte array
	 */
	protected static byte[] int2Byte(int[] src)
	{

		byte[] ret=new byte[src.length<<2]; // times 4
		int j=0;
		for(int i=0;i<src.length;i++)
		{
			ret[j++]=(byte) ((src[i] >>> 0) & 0xff);
			ret[j++]=(byte) ((src[i] >>> 8) & 0xff);
			ret[j++]=(byte) ((src[i] >>> 16) & 0xff);
			ret[j++]=(byte) ((src[i] >>> 24) & 0xff);
		}
		return ret;
	}
	
	/**
	 * Sort get sortet indexes.
	 *
	 * @param f the f
	 * @return the int[]
	 */
	public static int[] sortGetSortetIndexes(float[] f)
	{
		int[] ret=new int[f.length];
		for(int i=0;i<f.length;i++)
		{
			ret[i]=i;
		}
		for(int i=0;i<f.length-1;i++)for(int j=f.length-1;j>i;j--)if(f[ret[i]]>f[ret[j]])
		{
			int tmp=ret[i];
			ret[i]=ret[j];
			ret[j]=tmp;
		}
		return ret;
	}

	/*
	WHAT I WANTED! BUT COULDN'T GET
	public abstract static String createSQL(String table);
	public abstract static String getAllBySQL();
	 */
}
