/*********************************************************************************************
 * File:    Ap.java
 * Project: Indoor maps, Masterthesis
 * Version: 0.5b
 * Author:  Jon Villy Meidell
 * Org:     NTNU
 * 
 * 
 * Data Structure
 * An Access Point
 */
package no.ntnu.map.indoormaps;

import java.security.InvalidParameterException;
import java.util.AbstractList;
import java.util.ArrayList;

import javax.microedition.khronos.opengles.GL10;

import no.ntnu.map.indoormaps.Storage.SQLStorage.QueryBuilder;
import no.ntnu.map.indoormaps.Storage.SQLStorage.QueryBuilder.WrongTypeException;

import org.json.JSONObject;

import android.content.ContentResolver;
import android.content.ContentValues;
import android.content.Context;
import android.database.CharArrayBuffer;
import android.database.ContentObserver;
import android.database.Cursor;
import android.database.DataSetObserver;
import android.net.Uri;
import android.os.Bundle;

// TODO: Auto-generated Javadoc
/**
 * The Class Ap.
 */
public class Ap extends AbstractEntety
{

	/** TAG used with {@link IndMap#log(short, String, String)}. */
	public static final String TAG = "AP";

	/** The Database table name. Used in {@link Provider} and calls to it, {@link QueryBuilder}, ect*/
	public static final String TABLE_NAME = "ap";

	/** The Constant PARENT. */
	public static final String PARENT = "ap_id";

	/** The Constant BSSID. */
	public static final String BSSID = "mac";

	/** The Constant LEVELatNODE. */
	public static final String LEVELatNODE = "level";

	/** The Constant BSSID_SEPERATOR. */
	public static final String BSSID_SEPERATOR=":";

	/**
	 * The Class ApALC.
	 */
	public static class ApALC extends AbstractList<Ap> implements Cursor
	{

		/** The arr. */
		private ArrayList<Ap> arr = new ArrayList<Ap>();

		/** The channel. */
		private int[] level;

		/** The channel. */
		private int[]	channel;

		/** The pos. */
		private int pos=-1;

		/** The CHANNEL. */
		private final int _ID=0;

		/** The CHANNEL. */
		private final int	SSID=1;

		/** The CHANNEL. */
		private final int	BSSID=2;

		/** The CHANNEL. */
		private final int	LEVEL=3;

		/** The CHANNEL. */
		private final int	CHANNEL=4;

		/** The cols. */
		private final String[] cols=new String[]{Ap._ID, Ap.NAME, Ap.BSSID, "level", "freq"};

		/**
		 * Adds the.
		 *
		 * @param ap the ap
		 * @param level the level
		 * @param channel the channel
		 */
		public void add(Ap ap,int level, int channel)
		{
			arr.add(ap);
			int[] temp=new int[this.level.length+1];
			for(int i=0;i<this.level.length;i++)temp[i]=this.level[i];
			temp[this.level.length]=level;
			this.level=temp;

			temp=new int[this.channel.length+1];
			for(int i=0;i<this.channel.length;i++)temp[i]=this.channel[i];
			temp[this.level.length]=channel;
			this.channel=temp;

			for(int i=0;i<dso.size();i++)dso.get(i).onChanged();
		}

		/* (non-Javadoc)
		 * @see android.database.Cursor#close()
		 */
		public void close()
		{
			arr=null;
			level=new int[0];
			channel=new int[0];
			deactivate();
		}

		/* (non-Javadoc)
		 * @see android.database.Cursor#copyStringToBuffer(int, android.database.CharArrayBuffer)
		 */
		public void copyStringToBuffer(int columnIndex, CharArrayBuffer buffer){}

		/* (non-Javadoc)
		 * @see android.database.Cursor#deactivate()
		 */
		public void deactivate(){for(int i=0;i<dso.size();i++)dso.get(i).onInvalidated();}

		/* (non-Javadoc)
		 * @see android.database.Cursor#getBlob(int)
		 */
		public byte[] getBlob(int columnIndex){return null;}

		/* (non-Javadoc)
		 * @see android.database.Cursor#getColumnCount()
		 */
		public int getColumnCount(){return cols.length;}

		/* (non-Javadoc)
		 * @see android.database.Cursor#getColumnIndex(java.lang.String)
		 */
		public int getColumnIndex(String columnName)
		{
			for(int i=0;i<cols.length;i++)if(cols[i]==columnName)return i;
			return -1;
		}

		/* (non-Javadoc)
		 * @see android.database.Cursor#getColumnIndexOrThrow(java.lang.String)
		 */
		public int getColumnIndexOrThrow(String columnName) throws IllegalArgumentException
		{
			int r=getColumnIndex(columnName);
			if(r<0)throw new IllegalArgumentException("Column not found");
			return r;
		}

		/* (non-Javadoc)
		 * @see android.database.Cursor#getColumnName(int)
		 */
		public String getColumnName(int columnIndex){return cols[columnIndex];}

		/* (non-Javadoc)
		 * @see android.database.Cursor#getColumnNames()
		 */
		public String[] getColumnNames(){return cols;}

		/* (non-Javadoc)
		 * @see android.database.Cursor#getCount()
		 */
		public int getCount()
		{
			return arr.size();
		}

		/* (non-Javadoc)
		 * @see android.database.Cursor#getDouble(int)
		 */
		public double getDouble(int columnIndex)
		{
			throw new IllegalArgumentException("Column not of that type");
		}

		/* (non-Javadoc)
		 * @see android.database.Cursor#getExtras()
		 */
		public Bundle getExtras()
		{
			Bundle b=new Bundle();
			b.putString(cols[SSID], arr.get(pos).getName());
			b.putLong(cols[BSSID], arr.get(pos).getBssid());
			b.putInt(cols[LEVEL], level[pos]);
			b.putInt(cols[CHANNEL], channel[pos]);
			b.putLong(cols[_ID], arr.get(pos).getI());
			b.putInt(cols[_ID], arr.get(pos).getI());
			return b;
		}

		/* (non-Javadoc)
		 * @see android.database.Cursor#getFloat(int)
		 */
		public float getFloat(int columnIndex)
		{
			throw new IllegalArgumentException("Column not of that type");
		}

		/* (non-Javadoc)
		 * @see android.database.Cursor#getInt(int)
		 */
		public int getInt(int columnIndex)
		{
			switch (columnIndex)
			{
				case _ID:
					return arr.get(pos).getI();
				case LEVEL:
					return level[pos];
				case CHANNEL:
					return channel[pos];
				default:
					throw new IllegalArgumentException("Column not of that type");
			}
		}

		/* (non-Javadoc)
		 * @see android.database.Cursor#getLong(int)
		 */
		public long getLong(int columnIndex)
		{
			switch (columnIndex)
			{
				case _ID:
					return arr.get(pos).getI();
				case BSSID:
					return arr.get(pos).getBssid();
				default:
					throw new IllegalArgumentException("Column not of that type");
			}
		}

		/* (non-Javadoc)
		 * @see android.database.Cursor#getPosition()
		 */
		public int getPosition()
		{
			return pos;
		}

		/* (non-Javadoc)
		 * @see android.database.Cursor#getShort(int)
		 */
		public short getShort(int columnIndex)
		{
			throw new IllegalArgumentException("Column not of that type");
		}

		/* (non-Javadoc)
		 * @see android.database.Cursor#getString(int)
		 */
		public String getString(int columnIndex)
		{
			switch (columnIndex)
			{
				case SSID:
					return arr.get(pos).getName();
				default:
					throw new IllegalArgumentException("Column not of that type");
			}
		}

		/* (non-Javadoc)
		 * @see android.database.Cursor#getWantsAllOnMoveCalls()
		 */
		public boolean getWantsAllOnMoveCalls()
		{
			return false;
		}

		/* (non-Javadoc)
		 * @see android.database.Cursor#isAfterLast()
		 */
		public boolean isAfterLast()
		{
			return pos>=arr.size();
		}

		/* (non-Javadoc)
		 * @see android.database.Cursor#isBeforeFirst()
		 */
		public boolean isBeforeFirst()
		{
			return pos<0;
		}

		/* (non-Javadoc)
		 * @see android.database.Cursor#isClosed()
		 */
		public boolean isClosed()
		{
			return false;
		}

		/* (non-Javadoc)
		 * @see android.database.Cursor#isFirst()
		 */
		public boolean isFirst()
		{
			return pos==0;
		}

		/* (non-Javadoc)
		 * @see android.database.Cursor#isLast()
		 */
		public boolean isLast()
		{
			return pos==arr.size()-1;
		}

		/* (non-Javadoc)
		 * @see android.database.Cursor#isNull(int)
		 */
		public boolean isNull(int columnIndex)
		{
			switch (columnIndex)
			{
				case SSID:
					return arr.get(pos).getName()==null;
				case _ID:
					return arr.get(pos).getI()==0;
				default:
					throw new IllegalArgumentException("Column not of that type");
			}
		}

		/* (non-Javadoc)
		 * @see android.database.Cursor#move(int)
		 */
		public boolean move(int offset)
		{
			if(pos+offset>arr.size()-1||pos+offset<0)return false;
			pos+=offset;
			return true; 
		}

		/* (non-Javadoc)
		 * @see android.database.Cursor#moveToFirst()
		 */
		public boolean moveToFirst()
		{
			pos=0;
			return true;
		}

		/* (non-Javadoc)
		 * @see android.database.Cursor#moveToLast()
		 */
		public boolean moveToLast()
		{
			pos=arr.size()-1;
			return true;
		}

		/* (non-Javadoc)
		 * @see android.database.Cursor#moveToNext()
		 */
		public boolean moveToNext()
		{
			if(isAfterLast())return false;
			pos++;
			return false;
		}

		/* (non-Javadoc)
		 * @see android.database.Cursor#moveToPosition(int)
		 */
		public boolean moveToPosition(int position)
		{
			if(position>-1&&position<arr.size())
			{
				pos=position;
				return true;
			}
			return false;
		}

		/* (non-Javadoc)
		 * @see android.database.Cursor#moveToPrevious()
		 */
		public boolean moveToPrevious()
		{
			if(pos>0)
			{
				pos--;
				return true;
			}
			return false;
		}

		/** The co. */
		ArrayList<ContentObserver> co=new ArrayList<ContentObserver>(2);

		/* (non-Javadoc)
		 * @see android.database.Cursor#registerContentObserver(android.database.ContentObserver)
		 */
		public void registerContentObserver(ContentObserver observer){co.add(observer);}

		/** The dso. */
		ArrayList<DataSetObserver> dso=new ArrayList<DataSetObserver>(2);

		/* (non-Javadoc)
		 * @see android.database.Cursor#registerDataSetObserver(android.database.DataSetObserver)
		 */
		public void registerDataSetObserver(DataSetObserver observer){dso.add(observer);}

		/* (non-Javadoc)
		 * @see android.database.Cursor#requery()
		 */
		public boolean requery()
		{
			for(int i=0;i<co.size();i++)if(co.get(i).deliverSelfNotifications())co.get(i).onChange(true);
			for(int i=0;i<dso.size();i++)dso.get(i).onChanged();
			return true;
		}

		/* (non-Javadoc)
		 * @see android.database.Cursor#respond(android.os.Bundle)
		 */
		public Bundle respond(Bundle extras)
		{
			return Bundle.EMPTY;
		}

		/* (non-Javadoc)
		 * @see android.database.Cursor#setNotificationUri(android.content.ContentResolver, android.net.Uri)
		 */
		public void setNotificationUri(ContentResolver cr, Uri uri){}

		/* (non-Javadoc)
		 * @see android.database.Cursor#unregisterContentObserver(android.database.ContentObserver)
		 */
		public void unregisterContentObserver(ContentObserver observer){co.remove(observer);}

		/* (non-Javadoc)
		 * @see android.database.Cursor#unregisterDataSetObserver(android.database.DataSetObserver)
		 */
		public void unregisterDataSetObserver(DataSetObserver observer){dso.remove(observer);}

		/* (non-Javadoc)
		 * @see java.util.AbstractList#get(int)
		 */
		@Override
		public Ap get(int location)
		{
			return arr.get(location);
		}

		/* (non-Javadoc)
		 * @see java.util.AbstractCollection#size()
		 */
		@Override
		public int size()
		{
			return arr.size();
		}
	}

	/**
	 * The Class Entety.
	 */
	public static class Entety implements SQLEntety
	{

		/** The Constant PROJ_NODE_ALL. */
		public static final String[] PROJ_ALL	= {_ID, NAME, BSSID };
		/** The Constant PROJ_NODE_ALL. */
		public static final String[]	PROJ_NODE_ALL = { Ap._ID, Ap.NAME, Ap.BSSID, Ap.LEVELatNODE };

		/* (non-Javadoc)
		 * @see no.ntnu.map.indoormaps.AbstractEntety.SQLEntety#createTable()
		 */
		public String createTable() throws WrongTypeException
		{
			QueryBuilder qb = new QueryBuilder(QueryBuilder.CREATE);
			qb.setTable(TABLE_NAME);
			qb.addField(_ID, "INTEGER", true);
			qb.addField(NAME, "TEXT");
			qb.addField(BSSID, "TEXT");
			String ret = qb.getSqlString();
			qb = new QueryBuilder(QueryBuilder.CREATE);
			qb.setTable(TABLE_NAME + Node.TABLE_NAME);
			qb.addField(PARENT, "INTEGER", true);
			qb.addField(Node.PARENT, "INTEGER", true);
			qb.addField(LEVELatNODE, "INTEGER");
			ret += qb.getSqlString();
			return ret;
		}

		/* (non-Javadoc)
		 * @see no.ntnu.map.indoormaps.AbstractEntety.SQLEntety#dropTable()
		 */
		public String dropTable() throws WrongTypeException
		{
			return SQLEntety.DROPTABLESQL + TABLE_NAME;
		}

		/* (non-Javadoc)
		 * @see no.ntnu.map.indoormaps.AbstractEntety.SQLEntety#getAll()
		 */
		public String getAll() throws WrongTypeException
		{
			// Generate SQL-query to locate all rooms given the parent floor
			QueryBuilder qb = new QueryBuilder(QueryBuilder.SELECT);
			qb.addTable(TABLE_NAME);
			qb.addProjection(PROJ_ALL);
			qb.addOrdering(Ap._ID);
			return qb.getSqlString();
		}

		/* (non-Javadoc)
		 * @see no.ntnu.map.indoormaps.AbstractEntety.SQLEntety#getByParent(no.ntnu.map.indoormaps.AbstractEntety)
		 */
		public String getByParent(AbstractEntety parent) throws WrongTypeException
		{
			if(!(parent instanceof Node))throw new InvalidParameterException("An Ap's parent is supposed to be a Node");
			return getByParent(parent.getI());
		}

		/* (non-Javadoc)
		 * @see no.ntnu.map.indoormaps.AbstractEntety.SQLEntety#getByParent(int)
		 */
		public String getByParent(int parent_id) throws WrongTypeException
		{
			// Get Ap's by Node
			QueryBuilder qb = new QueryBuilder(QueryBuilder.SELECT);
			qb.addTable(TABLE_NAME);
			qb.addJoinCond(Ap._ID, QueryBuilder.EQUALS, Ap.PARENT);
			qb.addTable(TABLE_NAME + Node.TABLE_NAME);
			qb.addWhereCond(Node.PARENT, QueryBuilder.EQUALS, parent_id);
			qb.addProjection(PROJ_NODE_ALL);
			qb.addOrdering(Ap.LEVELatNODE, true);
			//			IndMap.log(IndMap.DEBUG, TAG, "Ap by node sql: "+qb.getSqlString());
			return qb.getSqlString();
		}

		/* (non-Javadoc)
		 * @see no.ntnu.map.indoormaps.AbstractEntety.SQLEntety#getShapes(no.ntnu.map.indoormaps.AbstractEntety, short)
		 */
		/**
		 * Gets the shapes.
		 *
		 * @param obj the obj
		 * @param match the match
		 * @return the shapes
		 * @throws WrongTypeException the wrong type exception
		 * @throws InvalidParameterException the invalid parameter exception
		 */
		public String getShapes(AbstractEntety obj, short match)
		throws WrongTypeException, InvalidParameterException
		{
			throw new InvalidParameterException("Ap's dont have shapes");
		}

		/* (non-Javadoc)
		 * @see no.ntnu.map.indoormaps.AbstractEntety.SQLEntety#getOne(int)
		 */
		public String getOne(int id) throws WrongTypeException
		{
			// Get Ap by BSSID
			QueryBuilder qb = new QueryBuilder(QueryBuilder.SELECT);
			qb.addTable(TABLE_NAME);
			qb.addProjection(PROJ_ALL);
			qb.addOrdering(Ap.BSSID);
			return qb.getSqlString();
		}
	}

	/* (non-Javadoc)
	 * @see no.ntnu.map.indoormaps.AbstractEntety#toInternal(android.content.Context)
	 */
	@Override
	void toInternal(Context c, int i)
	{
		ContentValues values=new ContentValues();
		values.put(_ID, _id);
		values.put(NAME,name);
		values.put(BSSID, bssid);
		try{c.getContentResolver().insert(Uri.withAppendedPath(Provider.CONTENT_URI,TABLE_NAME), values);}catch (Exception e) {
			IndMap.log(IndMap.ERROR, TAG, "Error save to internal "+e.toString());
		}
	}

	/**
	 * The bssid.
	 * @uml.property  name="bssid"
	 */
	private String bssid;

	/**
	 * The mac.
	 * @uml.property  name="mac"
	 */
	private long mac;

	/**
	 * The nodes.
	 * @uml.property  name="nodes"
	 * @uml.associationEnd  multiplicity="(0 -1)" inverse="aps:no.ntnu.map.indoormaps.Node"
	 */
	private ArrayList<Node>	nodes=new ArrayList<Node>(20);

	/**
	 * Instantiates a new ap.
	 *
	 * @param c A {@link Cursor} giving buildings from {@link Provider internal storage} 
	 */
	Ap(Cursor c)
	{
		_id=c.getInt(c.getColumnIndex(_ID));
		name=c.getString(c.getColumnIndex(NAME));
		bssid=c.getString(c.getColumnIndex(BSSID));
		mac=stringSSID2long(bssid, BSSID_SEPERATOR);
		if(IndMap.LOAD_INT_DEBUG)IndMap.log(IndMap.DEBUG, TAG, "In.New "+toString());
	}

	/**
	 * Gets the nodes.
	 *
	 * @return the nodes
	 */
	public ArrayList<Node> getNodes(){return nodes;}

	/**
	 * Adds the node.
	 *
	 * @param node the node
	 */
	public void addNode(Node node){if(!nodes.contains(node))nodes.add(node);}

	/**
	 * Gets the bssid.
	 *
	 * @return the bssid
	 */
	public long getBssid(){return mac;}

	/**
	 * Gets the bssid string.
	 *
	 * @return the bssid string
	 */
	public String getBssidString()
	{
		return bssid;
	}

	/**
	 * Instantiates a new ap.
	 *
	 * @param json the json
	 */
	Ap(JSONObject json)
	{
		try
		{
			_id = json.getInt(_ID);
			name = json.getString(NAME);
			bssid = json.getString(BSSID);
			mac=stringSSID2long(bssid, BSSID_SEPERATOR);
			if(IndMap.LOAD_EX_DEBUG)IndMap.log(IndMap.DEBUG, TAG, "Ex.New "+toString());
		} catch (Exception e)
		{
			IndMap.log(IndMap.ERROR, TAG, "Error conv from JSON to this " + TAG
					+ "" + e.toString());
			return;
		}
	}

	/**
	 * Instantiates a new ap.
	 *
	 * @param _id the _id
	 * @param ssid the ssid
	 * @param bssid the bssid
	 */
	protected Ap(int _id, String ssid,String bssid)
	{
		this._id=_id;
		name=ssid;
		this.bssid=bssid;
		mac=stringSSID2long(bssid, BSSID_SEPERATOR);
	}

	/* (non-Javadoc)
	 * @see no.ntnu.map.indoormaps.AbstractEntety#draw(javax.microedition.khronos.opengles.GL10, no.ntnu.map.indoormaps.World)
	 */
	@Override
	public void draw(GL10 gl, World world){}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString(){return name+" ("+bssid+" - "+mac+")";}

	/**
	 * String ssi d2long.
	 *
	 * @param ssid the ssid
	 * @param seperator the seperator
	 * @return the long
	 */
	public static long stringSSID2long(String ssid,String seperator)
	{
		return Long.parseLong(ssid.replaceAll(seperator,""), 16);
	}

	/**
	 * Gets the ap by bssid.
	 *
	 * @param elms the elms
	 * @param bssid the bssid
	 * @return the ap by bssid
	 */
	public static Ap getApByBSSID(Ap[] elms,long bssid)
	{
		int i=searchByBSSID(elms,0,elms.length,bssid);
		return (i<0?null:elms[i]);
	}

	/**
	 * Gets the index by bssid.
	 *
	 * @param elms the elms
	 * @param bssid the bssid
	 * @return the index by bssid
	 */
	public static int getIndexByBSSID(Ap[] elms,long bssid){return searchByBSSID(elms,0,elms.length,bssid);}

	/**
	 * Search by bssid.
	 *
	 * @param elms the elms
	 * @param min the min
	 * @param max the max
	 * @param bssid the bssid
	 * @return the int
	 */
	private static int searchByBSSID(Ap[] elms, int min, int max, long bssid)
	{
		//Recursive binary search
		//		IndMap.log(IndMap.DEBUG, TAG+"-binsearch", "Got "+elms.toString()+", "+elms.length+" elems , min: "+min+" max: "+max);
		int a=max-min;
		if(a < 1)return -1;
		int i = min + (a%2 == 1 ? a-1 : a) / 2;
		//		IndMap.log(IndMap.DEBUG, TAG+"-binsearch", "Center: "+i+ " "+bssid+" = "+elms[i].getBssid()+" ??");
		if(elms[i].getBssid()==bssid)return i;
		else if(a<2) return -1;
		else if(elms[i].getBssid()<bssid)return searchByBSSID(elms, i + 1, max, bssid);
		else return searchByBSSID(elms, min, i, bssid);
	}

	/**
	 * Contains by bssid.
	 *
	 * @param elms the elms
	 * @param bssid the bssid
	 * @return true, if successful
	 */
	public static boolean containsByBSSID(Ap[] elms,long bssid){return(getApByBSSID(elms, bssid)!=null);}

	/**
	 * Sort by id.
	 *
	 * @param elms the elms
	 * @return the ap[]
	 */
	public static Ap[] sortById(Ap[] elms)
	{
		Ap[] ret=new Ap[elms.length];
		int i;
		int max=0;
		for(i=0;i<elms.length;i++)
		{
			int id=elms[i].getI();
			//			if(id<0)throw new IndexOutOfBoundsException("An Ap can not have an negative id");
			if(id>max)max=id;
		}

		int[] map=new int[max+1];
		for(i=0;i<map.length;i++)
			map[i]=-1; //Init

		for(i=0;i<elms.length;i++)
		{
			int id=elms[i].getI();
			if(id<0)continue; //Filtering elements which not have an id. (those not known)
			map[id]=i; //Save index to element
		}
		max=0;
		for(i=0;i<map.length;i++)
		{
			if(max>=ret.length)throw new RuntimeException("Sry, some erreor with allocation for sorting method.");
			if(map[i]<0)continue;
			ret[max]=elms[map[i]]; //Load elements
			max++;
		}
		return ret;
	}
}


