/*********************************************************************************************
 * File:    Building.java
 * Project: Indoor maps, Masterthesis
 * Version: 0.5b
 * Author:  Jon Villy Meidell
 * Org:     NTNU
 * 
 * 
 * Data Structure
 * A Building
 */

package no.ntnu.map.indoormaps;

import java.security.InvalidParameterException;
import java.util.ArrayList;
import java.util.Iterator;

import javax.microedition.khronos.opengles.GL10;

import no.ntnu.map.indoormaps.World.Colour;
import no.ntnu.map.indoormaps.Storage.MysqlViaPHP;
import no.ntnu.map.indoormaps.Storage.SQLStorage.QueryBuilder;
import no.ntnu.map.indoormaps.Storage.SQLStorage.QueryBuilder.WrongTypeException;
import no.ntnu.map.indoormaps.World.LFEThread;

import org.json.JSONObject;

import android.content.ContentResolver;
import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.MatrixCursor;
import android.graphics.Color;
import android.net.Uri;
import android.provider.BaseColumns;
import android.widget.Spinner;

// TODO: Auto-generated Javadoc
/**
 * Part of datastructure -
 * A bulding.
 */
public class Building extends AbstractEntety
{
	/** TAG used with {@link IndMap#log(short, String, String)}. */
	public static final String TAG="Building"; //For debug

	/** The Database table name. Used in {@link Provider} and calls to it, {@link QueryBuilder}, ect*/
	public static final String TABLE_NAME = "building";

	/** Database field. */
	public static final String WIDTH="width";

	/** Database field. */
	public static final String LENGTH="length";

	/** Database field. */
	public static final String X="x";

	/** Database field. */
	public static final String Y="y";

	/** Database field. */
	public static final String Z="z";

	/** Database field. XY rotate */
	public static final String ROTATE="rotate";

	/** The parent key as foreign key. */
	public static final String PARENT="building_id";

	/**
	 * Generate SQL queries
	 * 
	 * Uses the table {@link Building#TABLE_NAME}<br>
	 * Fields:<ul>
	 * <li>{@link BaseColumns#_ID}</li>
	 * <li>{@link AbstractEntety#ID}</li>
	 * <li>{@link AbstractEntety#NAME}</li>
	 * <li>{@link Building#WIDTH}</li>
	 * <li>{@link Building#LENGTH}</li>
	 * <li>{@link Building#X}</li>
	 * <li>{@link Building#Y}</li>
	 * <li>{@link Building#Z}</li>
	 * <li>{@link Building#ROTATE}</li>
	 * <li>{@link AbstractEntety#WALL_BUF}</li>
	 * <li>{@link AbstractEntety#FLOOR_BUF}</li>
	 * </ul>.
	 */
	public static class Entety implements ShapeSQLEntety
	{

		/** Default Table projection in database. */
		public static final String[] PROJ_ALL={_ID,ID ,NAME ,  X,  Y,  Z,ROTATE,WIDTH,LENGTH,WALL_BUF,FLOOR_BUF};

		/** Emnty row in building {@link Spinner} using {@link MatrixCursor} in {@link NodeActivity#initNodelist}. */
		public static final Object[] SPINNER_TEXT_ALL={new Integer(0),"-","Outdoor","0","0","0","0",   "0",  "0",   null,     null};

		/**
		 * Creates the table.
		 *
		 * @return the string
		 * @throws WrongTypeException the wrong type exception
		 * @see no.ntnu.map.indoormaps.AbstractEntety.SQLEntety#createTable()
		 */
		public String createTable() throws WrongTypeException
		{
			QueryBuilder qb=new QueryBuilder(QueryBuilder.CREATE);
			qb.setTable(TABLE_NAME);
			qb.addField(_ID,    	"INTEGER", true);
			qb.addField(ID,     	"TEXT");
			qb.addField(NAME,   	"TEXT");
			qb.addField(WIDTH,  	"REAL");
			qb.addField(LENGTH, 	"REAL");
			qb.addField(X,      	"REAL");
			qb.addField(Y,      	"REAL");
			qb.addField(Z,      	"REAL");
			qb.addField(ROTATE, 	"REAL");
			qb.addField(WALL_BUF,	"BLOB");
			qb.addField(FLOOR_BUF,	"BLOB");
			return qb.getSqlString();
		}

		/**
		 * Gets the all.
		 *
		 * @return the all
		 * @throws WrongTypeException the wrong type exception
		 * @see no.ntnu.map.indoormaps.AbstractEntety.SQLEntety#getAll()
		 */
		public String getAll() throws WrongTypeException
		{
			//Generate SQL-query to locate all buildings
			QueryBuilder qb=new QueryBuilder(QueryBuilder.SELECT);
			qb.addTable(TABLE_NAME);
			qb.addProjection(PROJ_ALL);
			return qb.getSqlString();
		}

		/**
		 * Does nothing. A builing have children ({@link Floor} but no parents
		 *
		 * @param parent the parent
		 * @return the by parent
		 * @throws WrongTypeException the wrong type exception
		 * @see no.ntnu.map.indoormaps.AbstractEntety.SQLEntety#getByParent(no.ntnu.map.indoormaps.AbstractEntety)
		 */
		public String getByParent(AbstractEntety parent) throws WrongTypeException
		{
			return getByParent(0);
		}

		/**
		 * Does nothing. A builing have children ({@link Floor} but no parents
		 *
		 * @param parent_id the parent_id
		 * @return the by parent
		 * @throws WrongTypeException the wrong type exception
		 * @throws InvalidParameterException the invalid parameter exception
		 * @see no.ntnu.map.indoormaps.AbstractEntety.SQLEntety#getByParent(int)
		 */
		public String getByParent(int parent_id) throws WrongTypeException,InvalidParameterException {throw new InvalidParameterException("A building does not have a parent");}

		/**
		 * Drop table.
		 *
		 * @return the string
		 * @throws WrongTypeException the wrong type exception
		 * @see no.ntnu.map.indoormaps.AbstractEntety.SQLEntety#dropTable()
		 */
		public String dropTable() throws WrongTypeException
		{
			return SQLEntety.DROPTABLESQL+TABLE_NAME;
		}

		/**
		 * Gets the shapes.
		 *
		 * @param obj the obj
		 * @param match the match
		 * @return the shapes
		 * @throws WrongTypeException the wrong type exception
		 * @throws InvalidParameterException the invalid parameter exception
		 * @see no.ntnu.map.indoormaps.AbstractEntety.ShapeSQLEntety#getShapes(AbstractEntety, short)
		 */
		public String getShapes(AbstractEntety obj,short match) throws WrongTypeException, InvalidParameterException
		{
			if(obj instanceof Building)
			{
				QueryBuilder qb=new QueryBuilder(QueryBuilder.SELECT);
				String table=Building.TABLE_NAME+Shape.TABLE_NAME;
				qb.addTable(table);
				Shape.join2ParentAndPoint(qb,table);
				qb.addWhereCond(Building.PARENT, QueryBuilder.EQUALS, obj.getI());
				switch(match)
				{
					case Shape.WALL:
						qb.addWhereCond(Shape.TYPE, QueryBuilder.NOT_EQUAL, Shape.TYPE_BASE);
						break;
					case Shape.BASE:
						qb.addWhereCond(Shape.TYPE, QueryBuilder.EQUALS, Shape.TYPE_BASE);
						break;
				}
				//				IndMap.log(IndMap.DEBUG, TAG, "SQL for shape "+qb.getSqlString());
				return qb.getSqlString();
			}
			else throw new InvalidParameterException("Input is not a Building");
		}

		/**
		 * Gets the one.
		 *
		 * @param id the id
		 * @return the one
		 * @throws WrongTypeException the wrong type exception
		 * @see no.ntnu.map.indoormaps.AbstractEntety.SQLEntety#getOne(int)
		 */
		public String getOne(int id) throws WrongTypeException
		{
			//Generate SQL-query to locate one buildings
			QueryBuilder qb=new QueryBuilder(QueryBuilder.SELECT);
			qb.addTable(TABLE_NAME);
			qb.addProjection(PROJ_ALL);
			qb.addWhereCond(_ID, QueryBuilder.EQUALS, id);
			return qb.getSqlString();
		}
	}

	/**
	 * The floors.
	 * @uml.property  name="floors"
	 * @uml.associationEnd  multiplicity="(0 -1)" inverse="building:no.ntnu.map.indoormaps.Floor"
	 */
	private Floor[] floors=new Floor[0];

	/**
	 * The rooms.
	 * @uml.property  name="rooms"
	 * @uml.associationEnd  multiplicity="(0 -1)" inverse="building:no.ntnu.map.indoormaps.Room"
	 */
	private Room[] rooms=new Room[0];

	/**
	 * The position. Zero point of building in  {@link World#getCamera()  world coordinates} 
	 * @uml.property  name="position"
	 * @uml.associationEnd  multiplicity="(1 1)"
	 */
	private Point position;

	/**
	 * The xy rotate.
	 * @uml.property  name="xyRotate"
	 */
	private float xyRotate;

	/**
	 * The width of the building. Used to detect if user is inside buildings y-coord range
	 * @uml.property  name="width"
	 */
	private float width;
	/**
	 * The length of the building. Used to detect if user is inside buildings x-coord range
	 * @uml.property  name="length"
	 */
	private float length;

	/**
	 * Gets the floors of the builing.
	 * @return  the floors
	 * @uml.property  name="floors"
	 */
	public Floor[] getFloors(){return floors;}

	/**
	 * Gets the rooms of the builing.
	 * @return  the rooms
	 * @uml.property  name="rooms"
	 */
	public Room[] getRooms(){return rooms;}

	/**
	 * Gets the position of the builing.
	 * @return  the position
	 * @uml.property  name="position"
	 */
	public Point getPosition(){return position;}

	/**
	 * Gets the side rotate of the builing.
	 * @return  the xy rotate
	 * @uml.property  name="xyRotate"
	 */
	public float getXyRotate(){return xyRotate;}

	/**
	 * Gets the width of the builing. Used to detect if user is inside buildings y-coord range
	 * @return  the width
	 * @uml.property  name="width"
	 */
	public float getWidth(){return width;}

	/**
	 * Gets the length of the builing. Used to detect if user is inside buildings x-coord range
	 * @return  the length
	 * @uml.property  name="length"
	 */
	public float getLength(){return length;}

	/**
	 * Instantiates a new building.
	 *
	 * @param c A {@link Cursor} giving buildings from {@link Provider internal storage}
	 * @see Entety
	 */
	Building(Cursor c)
	{
		_id=c.getInt(c.getColumnIndex(_ID));
		id=c.getString(c.getColumnIndex(ID));
		name=c.getString(c.getColumnIndex(NAME));
		position=new Point(
				c.getFloat(c.getColumnIndex(X)), 
				c.getFloat(c.getColumnIndex(Y)), 
				c.getFloat(c.getColumnIndex(Z))
		);
		xyRotate=c.getFloat(c.getColumnIndex(ROTATE));
		width=c.getFloat(c.getColumnIndex(WIDTH));
		length=c.getFloat(c.getColumnIndex(LENGTH));

		vFloor=byte2Int(c.getBlob(c.getColumnIndex(FLOOR_BUF)));
		vWall=byte2Int(c.getBlob(c.getColumnIndex(WALL_BUF)));
		if(IndMap.LOAD_INT_DEBUG)IndMap.log(IndMap.DEBUG, TAG, "In.New "+toString());
	}

	/**
	 * Instantiates a new building from external datasource.
	 *
	 * @param json A {@link JSONObject} containing the given {@link Entety table fields}
	 */
	Building(JSONObject json)
	{
		try
		{
			_id=json.getInt(_ID);
			id=json.getString(ID);
			name=json.getString(NAME);
			position=new Point(MysqlViaPHP.toFloat(json.getDouble(X)), MysqlViaPHP.toFloat(json.getDouble(Y)), MysqlViaPHP.toFloat(json.getDouble(Z)));
			xyRotate=MysqlViaPHP.toFloat(json.getDouble(ROTATE));
			width=MysqlViaPHP.toFloat(json.getDouble(WIDTH));
			length=MysqlViaPHP.toFloat(json.getDouble(LENGTH));
			if(IndMap.LOAD_EX_DEBUG)IndMap.log(IndMap.DEBUG, TAG, "Ex.New "+toString());
		}
		catch (Exception e) {
			IndMap.log(IndMap.ERROR, TAG, "Error conv from JSON to this "+TAG+""+e.toString());
			return;
		}
	}

	/**
	 * To internal.
	 *
	 * @param c the c
	 * @see no.ntnu.map.indoormaps.AbstractEntety#toInternal(android.content.Context)
	 */
	@Override
	public void toInternal(Context c, int i)
	{
		ContentValues values=new ContentValues();
		values.put(_ID, _id);
		values.put(ID,id);
		values.put(NAME,name);
		values.put(WIDTH,width);
		values.put(LENGTH,length);
		values.put(X,position.x);
		values.put(Y,position.y);
		values.put(Z,position.z);
		values.put(ROTATE,xyRotate);
		values.put(WALL_BUF, int2Byte(vWall));
		values.put(FLOOR_BUF, int2Byte(vFloor));
		try{c.getContentResolver().insert(Uri.withAppendedPath(Provider.CONTENT_URI, TABLE_NAME), values);
		}catch (Exception e) {
			IndMap.log(IndMap.ERROR, TAG, "Error save to internal "+e.toString());
		}
	}

	/**
	 * Load its children from internal {@link Provider database}.
	 *
	 * @param app The application context, to be able to access the {@link ContentResolver}
	 * @return The found rooms in the children to link {@link Room rooms} and {@link Node nodes}
	 */
	ArrayList<Room> loadFromInternal(Context app)
	{
		ArrayList<Room> r = new ArrayList<Room>();
		ArrayList<Floor> f=new ArrayList<Floor>();

		generateBuffers();
		Cursor c=app.getContentResolver().query(
				Uri.withAppendedPath(Provider.CONTENT_URI, Building.TABLE_NAME+Floor.TABLE_NAME+"/"+_id),
				null, null, null, null);

		if(c.moveToFirst())
		{
			do
			{
				Floor fl=new Floor(c,this);
				f.add(fl);
				r.addAll(fl.loadFromInternal(app));
			}while(c.moveToNext());
		}

		floors=new Floor[f.size()];
		f.toArray(floors);
		rooms=new Room[r.size()];
		r.toArray(rooms);
		c.close();
		return r;
	}

	/**
	 * Load its children from external database.
	 *
	 * @param s The source to call new queries and get result from database.
	 * @param c The application context, to be able to access the {@link ContentResolver}
	 * @param hand To message loading progress and status back via {@link World.LFEThread} to {@link IndMap#hLoadExt}
	 * @return the array list
	 */
	ArrayList<Room> loadFromDatabase(MysqlViaPHP s,Context c,LFEThread hand)
	{
		ShapeSQLEntety fSQL=new Floor.Entety(),bSQL=new Building.Entety();

		hand.setMessage("Generating shapes");
		hand.setMessage(c.getResources().getString(R.string.shapes));
		try
		{
			s.executeQuery(bSQL.getShapes(this, Shape.BASE));
			floorFromDatabase(s,0); //TODO Kanskje vil ha faktisk posisjon i nyere versjon?
			length=getMaxLength();
			width=getMaxWidth();
			s.executeQuery(bSQL.getShapes(this, Shape.WALL));
			wallFromDatabase(s,(int) position.z);

			toInternal(c, 0);

			generateBuffers();
		}
		catch (Exception e) {
			IndMap.log(IndMap.ERROR, TAG, "Get Vertexes "+e.toString());
			return null;
		}

		hand.doIncr((int)(hand.getIncr()/20));
		try
		{
			ArrayList<Floor> f=new ArrayList<Floor>();

			//Gets the sql from the Floor class itself
			hand.setMessage("Querying database");
			s.executeQuery(fSQL.getByParent(this._id));
			hand.setMessage("Processing");
			//Adds all floors
			while(s.next()) f.add(new Floor(s.getJSONObject(),this));
			Iterator<Floor> i=f.iterator();
			ArrayList<Room> r = new ArrayList<Room>(100);

			//Makes the floors find their rooms
			byte lev=0;
			while(i.hasNext())
			{
				Floor fl=i.next();
				hand.setMessage(c.getResources().getString(R.string.floor)+" "+fl.getName());				
				r.addAll(fl.loadFromDatabase(s,c,lev));
				lev++;
				hand.doIncr((int)(hand.getIncr()*4/20/f.size()));
			}
			floors=new Floor[f.size()];
			f.toArray(floors);
			rooms=new Room[r.size()];
			r.toArray(rooms);
			return r;
		}
		catch (Exception e) {
			IndMap.log(IndMap.ERROR, TAG, "Create Floors "+e.toString());
			return null;
		}
	}

	/**
	 * To string.
	 *
	 * @return the string
	 * @see java.lang.Object#toString()
	 */
	public String toString()
	{
		return id+" - "+name+" "+position.toString();
	}

	/**
	 * Draw.
	 *
	 * @param gl the gl
	 * @param w the world
	 * @see no.ntnu.map.indoormaps.AbstractEntety#draw(javax.microedition.khronos.opengles.GL10, no.ntnu.map.indoormaps.World)
	 */
	@Override
	public void draw(GL10 gl, World w)
	{
		gl.glPushMatrix();
		gl.glTranslatef(position.y, position.z, -position.x);
		gl.glRotatef(xyRotate, 0f, 1.0f, 0f);

		Colour c;
		Building at=w.getAtBuilding();
		if(!equals(at))
		{
			c=new Colour(0, 255, 160, 255);
			draw(gl, GL10.GL_TRIANGLES, vWallBuffer, num_vWall, Color.argb(c.alpha, c.red, c.green, c.blue));

			c=World.COL_NOT_AT_FLOOR;

			if(w.isMode_draw2D())
				draw(gl, GL10.GL_TRIANGLES, vFloorBuffer, num_vFloor, Color.argb(c.alpha, c.red, c.green, c.blue));
		}

		for(Floor f:floors)f.draw(gl, w);

		gl.glPopMatrix();
	}
}
