/*********************************************************************************************
 * File:    Floor.java
 * Project: Indoor maps, Masterthesis
 * Version: 0.5b
 * Author:  Jon Villy Meidell
 * Org:     NTNU
 * 
 * 
 * Data Structure
 * A Floor
 */

package no.ntnu.map.indoormaps;

import java.security.InvalidParameterException;
import java.util.ArrayList;
import java.util.Iterator;

import javax.microedition.khronos.opengles.GL10;

import no.ntnu.map.indoormaps.Storage.MysqlViaPHP;
import no.ntnu.map.indoormaps.Storage.SQLStorage.QueryBuilder;
import no.ntnu.map.indoormaps.Storage.SQLStorage.QueryBuilder.WrongTypeException;
import no.ntnu.map.indoormaps.World.Colour;

import org.json.JSONObject;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.graphics.Color;
import android.net.Uri;

// TODO: Auto-generated Javadoc
/**
 * The Class Floor.
 */
public class Floor extends AbstractEntety
{

	/** TAG used with {@link IndMap#log(short, String, String)}. */
	public static final String TAG="Floor";

	/** The Database table name. Used in {@link Provider} and calls to it, {@link QueryBuilder}, ect*/
	public static final String TABLE_NAME = "floor";
	public static final String PRI = "pri";

	/** The parent key as foreign key. */
	public static final String PARENT="floor_id";

	/**
	 * The Class Entety.
	 */
	public static class Entety implements ShapeSQLEntety
	{

		/** The Constant SPINNER_TEXT_ALL. */
		public static final String[] PROJ_ALL={_ID,ID, NAME,PRI,WALL_BUF,FLOOR_BUF};
		/** The Constant SPINNER_TEXT_ALL. */
		public static final String[]	SPINNER_TEXT_ALL={"0","-","All","","",""};

		/* (non-Javadoc)
		 * @see no.ntnu.map.indoormaps.AbstractEntety.SQLEntety#createTable()
		 */
		public String createTable() throws WrongTypeException
		{
			QueryBuilder qb=new QueryBuilder(QueryBuilder.CREATE);
			qb.setTable(TABLE_NAME);
			qb.addField(_ID, "INTEGER", true);
			qb.addField(ID, "TEXT");
			qb.addField(NAME, "TEXT");
			qb.addField(PRI, "INTEGER");
			qb.addField(Building.PARENT, "INTEGER");
			qb.addField(WALL_BUF,	"BLOB");
			qb.addField(FLOOR_BUF,	"BLOB");
			return qb.getSqlString();
		}

		/* (non-Javadoc)
		 * @see no.ntnu.map.indoormaps.AbstractEntety.SQLEntety#getAll()
		 */
		public String getAll() throws WrongTypeException
		{
			//Generate SQL-query to locate all floors
			QueryBuilder qb=new QueryBuilder(QueryBuilder.SELECT);
			qb.addTable(TABLE_NAME);
			qb.addProjection(PROJ_ALL);
			qb.addOrdering(Building.PARENT);
			qb.addOrdering(PRI);
			return qb.getSqlString();
		}

		/* (non-Javadoc)
		 * @see no.ntnu.map.indoormaps.AbstractEntety.SQLEntety#getByParent(no.ntnu.map.indoormaps.AbstractEntety)
		 */
		public String getByParent(AbstractEntety parent) throws WrongTypeException
		{
			if(!(parent instanceof Building))throw new InvalidParameterException("A Floor's parent is supposed to be a Building");
			return getByParent(parent.getI());
		}

		/* (non-Javadoc)
		 * @see no.ntnu.map.indoormaps.AbstractEntety.SQLEntety#getByParent(int)
		 */
		public String getByParent(int parent_id) throws WrongTypeException
		{
			//Generate SQL-query to locate all floors given the parent building
			QueryBuilder qb=new QueryBuilder(QueryBuilder.SELECT);
			qb.addTable(TABLE_NAME);
			qb.addProjection(PROJ_ALL);
			qb.addWhereCond(Building.PARENT, QueryBuilder.EQUALS, parent_id);
			qb.addOrdering(PRI);
			return qb.getSqlString();
		}

		/* (non-Javadoc)
		 * @see no.ntnu.map.indoormaps.AbstractEntety.SQLEntety#dropTable()
		 */
		public String dropTable() throws WrongTypeException
		{
			return SQLEntety.DROPTABLESQL+TABLE_NAME;
		}

		/* (non-Javadoc)
		 * @see no.ntnu.map.indoormaps.AbstractEntety.SQLEntety#getShapes(no.ntnu.map.indoormaps.AbstractEntety, short)
		 */
		public String getShapes(AbstractEntety obj, short match) throws WrongTypeException, InvalidParameterException
		{
			if(obj instanceof Floor)
			{
				QueryBuilder qb=new QueryBuilder(QueryBuilder.SELECT);
				String table=Floor.TABLE_NAME+Shape.TABLE_NAME;
				qb.addTable(table);
				Shape.join2ParentAndPoint(qb,table);
				qb.addWhereCond(Floor.PARENT, QueryBuilder.EQUALS, obj.getI());
				switch(match)
				{
					case Shape.WALL:
						qb.addWhereCond(Shape.TYPE, QueryBuilder.NOT_EQUAL, Shape.TYPE_BASE);
						break;
					case Shape.BASE:
						qb.addWhereCond(Shape.TYPE, QueryBuilder.EQUALS, Shape.TYPE_BASE);
						break;
				}
				return qb.getSqlString();
			}
			else throw new InvalidParameterException("Input is not a Floor");
		}

		/* (non-Javadoc)
		 * @see no.ntnu.map.indoormaps.AbstractEntety.SQLEntety#getOne(int)
		 */
		public String getOne(int id) throws WrongTypeException
		{
			//Generate SQL-query to locate one floors
			QueryBuilder qb=new QueryBuilder(QueryBuilder.SELECT);
			qb.addTable(TABLE_NAME);
			qb.addProjection(PROJ_ALL);
			qb.addWhereCond(_ID, QueryBuilder.EQUALS, id);
			return qb.getSqlString();
		}
	}

	/**
	 * Load from internal.
	 *
	 * @param app the app
	 * @return the array list
	 */
	ArrayList<Room> loadFromInternal(Context app)
	{
		ArrayList<Room> r=new ArrayList<Room>();

		generateBuffers();
		Cursor c=app.getContentResolver().query(
				Uri.withAppendedPath(Provider.CONTENT_URI, Floor.TABLE_NAME+Room.TABLE_NAME+"/"+_id),
				null, null, null, null);
		if(c.moveToFirst())
		{
			do{
				Room ro=new Room(c,this,this.building);
				r.add(ro);
				ro.generateBuffers();
			}while(c.moveToNext());
		}
		rooms=new Room[r.size()];
		r.toArray(rooms);
		c.close();
		return r;
	}

	/* (non-Javadoc)
	 * @see no.ntnu.map.indoormaps.AbstractEntety#toInternal(android.content.Context)
	 */
	@Override
	void toInternal(Context c, int i)
	{
		ContentValues values=new ContentValues();
		values.put(_ID, _id);
		values.put(ID, id);
		values.put(NAME,name);
		values.put(PRI, i);
		values.put(Building.PARENT,building.getI());
		values.put(WALL_BUF, int2Byte(vWall));
		values.put(FLOOR_BUF, int2Byte(vFloor));

		try{c.getContentResolver().insert(Uri.withAppendedPath(Provider.CONTENT_URI, TABLE_NAME), values);}catch (Exception e) {
			IndMap.log(IndMap.ERROR, TAG, "Error save to internal "+e.toString());
		}
	}

	/**
	 * The building.
	 * @uml.property  name="building"
	 * @uml.associationEnd  multiplicity="(1 1)" inverse="floors:no.ntnu.map.indoormaps.Building"
	 */
	private Building building;

	/**
	 * The rooms.
	 * @uml.property  name="rooms"
	 * @uml.associationEnd  multiplicity="(0 -1)" inverse="floor:no.ntnu.map.indoormaps.Room"
	 */
	private Room[] rooms;

	/**
	 * Instantiates a new floor.
	 *
	 * @param c A {@link Cursor} giving buildings from {@link Provider internal storage} 
	 * @param parent the parent
	 */
	Floor(Cursor c,Building parent)
	{
		building=parent;
		_id=c.getInt(c.getColumnIndex(_ID));
		id=c.getString(c.getColumnIndex(ID));
		name=c.getString(c.getColumnIndex(NAME));

		vFloor=byte2Int(c.getBlob(c.getColumnIndex(FLOOR_BUF)));
		vWall=byte2Int(c.getBlob(c.getColumnIndex(WALL_BUF)));
		if(IndMap.LOAD_INT_DEBUG)IndMap.log(IndMap.DEBUG, TAG, "In.New "+toString());
	}

	/**
	 * Gets the building.
	 * @return  the building
	 * @uml.property  name="building"
	 */
	public Building getBuilding(){return building;}

	/**
	 * Sets the building.
	 * @param building  the new building
	 * @uml.property  name="building"
	 */
	public void setBuilding(Building building){this.building = building;}

	/**
	 * Sets the rooms.
	 * @param rooms  the new rooms
	 * @uml.property  name="rooms"
	 */
	public void setRooms(Room[] rooms){this.rooms = rooms;}

	/**
	 * Gets the rooms.
	 * @return  the rooms
	 * @uml.property  name="rooms"
	 */
	public Room[] getRooms(){return rooms;}

	/**
	 * Instantiates a new floor.
	 *
	 * @param json the json
	 * @param parent the parent
	 */
	Floor(JSONObject json,Building parent)
	{
		try
		{
			//Generate itself from the result of a sql-query
			building=parent;
			_id=json.getInt(_ID);
			id=json.getString(ID);
			name=json.getString(NAME);
			if(IndMap.LOAD_EX_DEBUG)IndMap.log(IndMap.DEBUG, TAG, "Ex.New "+toString());
		}
		catch (Exception e) {
			IndMap.log(IndMap.ERROR, TAG, "Error conv from JSON to this "+TAG+""+e.toString());
			return;
		}
	}

	/**
	 * Load from database.
	 *
	 * @param s the s
	 * @param c the c
	 * @param level the level
	 * @return the array list
	 */
	ArrayList<Room> loadFromDatabase(MysqlViaPHP s,Context c,byte level)
	{
		ShapeSQLEntety fSQL=new Floor.Entety(),rSQL=new Room.Entety();
		//TOTO Get outerwall of building from external, generate vertex-buffer, and save to internal
		try
		{
			s.executeQuery(fSQL.getShapes(this, Shape.BASE));
			floorFromDatabase(s,level*(Shape.WALL_HEIGHT+Shape.SPACE_BETWEEN_FLOORS));
			s.executeQuery(fSQL.getShapes(this, Shape.WALL));
			wallFromDatabase(s,level*(Shape.WALL_HEIGHT+Shape.SPACE_BETWEEN_FLOORS));

			toInternal(c,level+1);
			generateBuffers();
		}
		catch (Exception e) {
			IndMap.log(IndMap.ERROR, TAG, "Get Vertexes "+e.toString());
			return null;
		}
		try
		{
			ArrayList<Room> r=new ArrayList<Room>(20);

			//Gets the sql from the Room class itself
			s.executeQuery(rSQL.getByParent(this._id));
			//Adds all rooms
			while(s.next())r.add(new Room(s.getJSONObject(),this,this.building));

			//Make rooms load their vertexes
			Iterator<Room> i=r.iterator();
			while(i.hasNext())i.next().loadFromDatabase(s,c,level*(Shape.WALL_HEIGHT+Shape.SPACE_BETWEEN_FLOORS));

			//Return the rooms for the building
			rooms=new Room[r.size()];
			r.toArray(rooms);
			return r;
		}
		catch (Exception e) {
			IndMap.log(IndMap.ERROR, TAG, "Create Rooms "+e.toString());
			return null;
		}
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString(){return id+" Etasje: "+name;}

	/* (non-Javadoc)
	 * @see no.ntnu.map.indoormaps.AbstractEntety#draw(javax.microedition.khronos.opengles.GL10, no.ntnu.map.indoormaps.World)
	 */
	@Override
	public void draw(GL10 gl, World w)
	{
		Floor at=w.getAtFloor();
		
		if(!w.isMode_draw2D()||(w.isMode_draw2D()&&equals(at)))
		{
			Colour c=World.COL_NULL_AT_WALL;
			if(equals(at))c=World.COL_AT_WALL;
			else if(at!=null)c=World.COL_NOT_AT_WALL;
			
			draw(gl, GL10.GL_TRIANGLES, vWallBuffer, num_vWall, Color.argb(c.alpha*(w.isMode_draw2D()?1/2:1), c.red, c.green, c.blue));
			
			if(equals(at))c=World.COL_AT_FLOOR;
			else c=World.COL_NOT_AT_FLOOR;
			
			draw(gl, GL10.GL_TRIANGLES, vFloorBuffer, num_vFloor, Color.argb(c.alpha, c.red, c.green, c.blue));
			
			for(Room r:rooms)r.draw(gl, w);
		}
	}
}
