/*********************************************************************************************
 * File:    IndMapRenderer.java
 * Project: Indoor maps, Masterthesis
 * Version: 0.5b
 * Author:  Jon Villy Meidell
 * Org:     NTNU
 * 
 * 
 * Grafical User Interface
 * The renderer that is loaded to a GLSurfaceView, in IndMap.java
 */

package no.ntnu.map.indoormaps;

import android.opengl.GLSurfaceView;
import android.opengl.GLU;

import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;

// TODO: Auto-generated Javadoc
/**
 * Open GL renderer.
 */
public class IndMapRenderer implements GLSurfaceView.Renderer 
{

	/** The PRO j_ angle. */
	public static float PROJ_ANGLE=80f;
	/**
	 * The world. Needed to call  {@link World#draw(GL10)} 
	 * @uml.property  name="world"
	 * @uml.associationEnd  multiplicity="(1 1)"
	 */
	private World world;

	/**
	 * Instantiates a new ind map renderer.
	 *
	 * @param world the world
	 */
	public IndMapRenderer(World world)
	{
		this.world = world;
	}

	/**
	 * Called each frame. Makes the call to {@link World#draw(GL10)}
	 *
	 * @param gl the gl
	 * @see android.opengl.GLSurfaceView.Renderer#onDrawFrame(javax.microedition.khronos.opengles.GL10)
	 */
	public void onDrawFrame(GL10 gl)
	{
		gl.glClear(GL10.GL_COLOR_BUFFER_BIT | GL10.GL_DEPTH_BUFFER_BIT);
		gl.glLoadIdentity();

		gl.glPushMatrix();

		//		gl.glLightfv(GL10.GL_LIGHT0, GL10.GL_POSITION, new float[]{0.0f,100.0f,0.0f,1.0f}, 0);
		//		gl.glLightfv(GL10.GL_LIGHT0, GL10.GL_AMBIENT, new float[]{0.5f,0.5f,0.5f,1.0f}, 0);
		//		gl.glLightfv(GL10.GL_LIGHT0, GL10.GL_DIFFUSE, new float[]{0.8f,0.8f,0.8f,1.0f}, 0);
		//		gl.glLightfv(GL10.GL_LIGHT0, GL10.GL_SPECULAR, new float[]{1.0f,1.0f,1.0f,1.0f}, 0);
		//		gl.glLightf(GL10.GL_LIGHT0, GL10.GL_SPOT_CUTOFF, 108.0f);
		//		gl.glLightf(GL10.GL_LIGHT0, GL10.GL_SPOT_EXPONENT, 30.0f);

		world.draw(gl);

		gl.glPopMatrix();
	}

	/**
	 * <h2> Init Open GL</h2>
	 * 
	 * Set up GL properties, viewport, projectionmode ect.
	 *
	 * @param gl the gl
	 * @param width the width
	 * @param height the height
	 * @see android.opengl.GLSurfaceView.Renderer#onSurfaceChanged(javax.microedition.khronos.opengles.GL10, int, int)
	 */
	public void onSurfaceChanged(GL10 gl, int width, int height)
	{
		gl.glEnable(GL10.GL_COLOR_MATERIAL);
		//		gl.glEnable(GL10.GL_LIGHTING);
		//		gl.glEnable(GL10.GL_LIGHT0);
		//		gl.glEnable(GL10.GL_CULL_FACE);
		gl.glEnable(GL10.GL_DEPTH_BUFFER_BIT);
		gl.glEnable(GL10.GL_DEPTH_TEST);
		gl.glDepthFunc(GL10.GL_LEQUAL);
		gl.glEnable(GL10.GL_BLEND);
		gl.glBlendFunc(GL10.GL_ONE, GL10.GL_ONE_MINUS_SRC_ALPHA);
		gl.glClearColor(0.4f,0.4f,0.6f,1f);
		gl.glShadeModel(GL10.GL_SMOOTH);

		gl.glViewport(0, 0, width, height);

		gl.glMatrixMode(GL10.GL_PROJECTION);
		gl.glLoadIdentity();
		GLU.gluPerspective(gl, PROJ_ANGLE, ( (float)width / (float)height), 0.2f, 3000.0f);

		gl.glMatrixMode(GL10.GL_MODELVIEW);
		gl.glLoadIdentity();		
	}

	/**
	 * Not used yet (Do not have any textures).
	 *
	 * @param gl the gl
	 * @param config the config
	 */
	public void onSurfaceCreated(GL10 gl, EGLConfig config){}
}
