/*********************************************************************************************
 * Project: Indoor maps, Masterthesis
 * Version: 1.0b
 * Author:  Jon Villy Meidell
 * Org:     NTNU
 */

/*

        <provider android:name="IndMapProvider"
            android:authorities="no.ntnu.map.indoormaps.IndMapProvider"
        />
 */

package no.ntnu.map.indoormaps;

import java.util.ArrayList;
import java.util.Iterator;

import no.ntnu.map.indoormaps.AbstractEntety.SQLEntety;
import no.ntnu.map.indoormaps.Node.NodeSQLEntety;
import no.ntnu.map.indoormaps.Storage.InternalStorage;

import android.content.ContentProvider;
import android.content.ContentValues;
import android.content.Context;
import android.content.UriMatcher;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.net.Uri;

// TODO: Auto-generated Javadoc
/**
 * The Class Provider.
 */
public class Provider extends ContentProvider
{

	/** The Constant TAG. */
	public static final String AUTHORITY="no.ntnu.map.indoormaps.Provider";

	/** The Constant TAG. */
	public static final String	TAG="Provider";

	/** The Constant CONTENT_URI. */
	public static final Uri CONTENT_URI = Uri.parse("content://" + AUTHORITY);

	//Load default interfaces for functions implemented by AbstractEntety.SQLEntety for each entety
	/**
	 * The a ent.
	 * @uml.property  name="bEnt"
	 * @uml.associationEnd  multiplicity="(1 1)"
	 */
	private SQLEntety
	bEnt=new Building.Entety();

	/**
	 * The a ent.
	 * @uml.property  name="fEnt"
	 * @uml.associationEnd  multiplicity="(1 1)"
	 */
	private SQLEntety	fEnt=new Floor.Entety();

	/**
	 * The a ent.
	 * @uml.property  name="rEnt"
	 * @uml.associationEnd  multiplicity="(1 1)"
	 */
	private SQLEntety	rEnt=new Room.Entety();

	/**
	 * The a ent.
	 * @uml.property  name="aEnt"
	 * @uml.associationEnd  multiplicity="(1 1)"
	 */
	private SQLEntety	aEnt=new Ap.Entety();

	/**
	 * The n ent.
	 * @uml.property  name="nEnt"
	 * @uml.associationEnd  multiplicity="(1 1)"
	 */
	private NodeSQLEntety nEnt=new Node.Entety();

	//private WifiManager wifi = null;

	/**
	 * The m open helper.
	 * @uml.property  name="mOpenHelper"
	 * @uml.associationEnd  
	 */
	private DatabaseHelper mOpenHelper;

	/**
	 * The Class DatabaseHelper.
	 */
	private static class DatabaseHelper extends SQLiteOpenHelper implements InternalStorage
	{

		/** The Constant DATABASE_NAME. */
		private static final String DATABASE_NAME = "indoor_maps.db";

		/** The Constant DATABASE_VERSION. */
		private static final int DATABASE_VERSION = 1;

		/** The enteties. */
		private ArrayList<SQLEntety> enteties;

		/**
		 * Instantiates a new database helper.
		 *
		 * @param context the context
		 * @param enteties the enteties
		 */
		DatabaseHelper(Context context, ArrayList<SQLEntety> enteties)
		{
			super(context, DATABASE_NAME, null, DATABASE_VERSION);
			this.enteties=enteties;
		}

		/* (non-Javadoc)
		 * @see android.database.sqlite.SQLiteOpenHelper#onCreate(android.database.sqlite.SQLiteDatabase)
		 */
		@Override
		public void onCreate(SQLiteDatabase db)
		{
			Iterator<SQLEntety> i=enteties.iterator();
			IndMap.log(IndMap.DEBUG, TAG, "Created SQLite Tables:");
			while(i.hasNext())
			{
				try
				{
					String[] sql=i.next().createTable().split(";");
					for(int j=0;j<sql.length;j++)db.execSQL(sql[j]+";");
				}
				catch (Exception e) {
					IndMap.log(IndMap.ERROR, TAG, "createTable-"+e.toString());
					return;
				}
			}
		}

		/* (non-Javadoc)
		 * @see android.database.sqlite.SQLiteOpenHelper#onUpgrade(android.database.sqlite.SQLiteDatabase, int, int)
		 */
		@Override
		public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion)
		{
			IndMap.log(IndMap.WARNING, TAG, "Update database from version " +oldVersion+" to "+newVersion+" destroying old data");
			Iterator<SQLEntety> i=enteties.iterator();
			while(i.hasNext())
			{
				try{db.execSQL(i.next().dropTable());}
				catch (Exception e) {
					IndMap.log(IndMap.ERROR, TAG, "onUpgrade-"+e.toString());
				}
			}
			onCreate(db);
		}
	}

	//TODO lage public static getUri(int tables) samt oppslagstabell mellom int og string, som utgjr uri. Iterere gjennom int'er for  generere alle uri-matcher, i stedet for tungvint vanlig
	/** The Constant ROOM. */
	public static final int ID=1;

	/** The Constant ROOM. */
	public static final int	NODE=3;

	/** The Constant ROOM. */
	public static final int	ROUTE=9;

	/** The Constant ROOM. */
	public static final int	AP=15;

	/** The Constant ROOM. */
	public static final int	BUILDING=32;

	/** The Constant ROOM. */
	public static final int	FLOOR=64;

	/** The Constant ROOM. */
	public static final int	ROOM=128;

	/*
	 * Requirements for URI's:
	 *  - ... + ID
	 *  - Building							(find all buildings)
	 *  - Building + Floor + ID				(find floors given builing)
	 *  - Floor + Room + ID					(find rooms given floor)
	 *  - Floor + ID + Room + ID			(find match in FloorRoom table)			= 1 + 1
	 *  - Node								(find all nodes
	 *  - Node + Node + ID					(find child node given parent)
	 *  - Node + ID + Node + ID				(find match in NodeNode table)			= 1 + 1
	 *  - Ap								(find all AP's)
	 *  - Ap + Node + ID					(find all Ap's given node)
	 *  - Route								(find all routes
	 *  - Route + ID						(one route)
	 *  - Route + Node + ID					(find all nodes (start, via,end nodes) by route
	 *  - Route + ID + Node + ID			(find match in RouteNode table)
	 *  
	 *  - Building + ID + FLOOR + NODE      (find nodes given building id)
	 */
	/** The Constant sURIMatcher. */
	public static final UriMatcher sURIMatcher = buildUriMatcher();

	/**
	 * Builds the uri matcher.
	 *
	 * @return the uri matcher
	 */
	private static UriMatcher buildUriMatcher()
	{
		UriMatcher matcher =  new UriMatcher(UriMatcher.NO_MATCH);
		matcher.addURI(AUTHORITY, Building.TABLE_NAME, BUILDING);
		matcher.addURI(AUTHORITY, Building.TABLE_NAME+"/#", BUILDING+ID);

		matcher.addURI(AUTHORITY, Floor.TABLE_NAME, FLOOR);
		matcher.addURI(AUTHORITY, Floor.TABLE_NAME+"/#", FLOOR+ID);

		matcher.addURI(AUTHORITY, Room.TABLE_NAME, ROOM);
		matcher.addURI(AUTHORITY, Room.TABLE_NAME+"/#", ROOM+ID);

		//matcher.addURI(AUTHORITY, Building.TABLE_NAME+Floor.TABLE_NAME, FLOOR+BUILDING);
		matcher.addURI(AUTHORITY, Building.TABLE_NAME+Floor.TABLE_NAME+"/#", BUILDING+FLOOR+ID);
		matcher.addURI(AUTHORITY, Building.TABLE_NAME+"/#/"+Floor.TABLE_NAME+"/#", BUILDING+ID+FLOOR+ID);

		//matcher.addURI(AUTHORITY, Room.TABLE_NAME+Floor.TABLE_NAME, ROOM+FLOOR);
		matcher.addURI(AUTHORITY, Floor.TABLE_NAME+Room.TABLE_NAME+"/#", ROOM+FLOOR+ID);
		matcher.addURI(AUTHORITY, Floor.TABLE_NAME+"/#/"+Room.TABLE_NAME+"/#", FLOOR+ID+ROOM+ID);

		matcher.addURI(AUTHORITY, Node.TABLE_NAME, NODE);
		matcher.addURI(AUTHORITY, Node.TABLE_NAME+"null", BUILDING-NODE);
		matcher.addURI(AUTHORITY, Node.TABLE_NAME+"/#", NODE+ID);

		matcher.addURI(AUTHORITY, Node.TABLE_NAME+Building.TABLE_NAME+"/#", NODE+BUILDING+ID);
		matcher.addURI(AUTHORITY, Node.TABLE_NAME+Floor.TABLE_NAME+"/#", NODE+FLOOR+ID);
		matcher.addURI(AUTHORITY, Node.TABLE_NAME+Room.TABLE_NAME+"/#", NODE+ROOM+ID);

		matcher.addURI(AUTHORITY, Node.TABLE_NAME+Floor.TABLE_NAME+"/#/"+Room.TABLE_NAME+"/#", NODE+FLOOR+ID+ROOM+ID);
		matcher.addURI(AUTHORITY, Node.TABLE_NAME+Node.TABLE_NAME, NODE+NODE);
		matcher.addURI(AUTHORITY, Node.TABLE_NAME+"/#/"+Node.TABLE_NAME+"/#", NODE+ID+NODE+ID);
		matcher.addURI(AUTHORITY, Node.TABLE_NAME+Node.TABLE_NAME+"/#", NODE+NODE+ID);

		matcher.addURI(AUTHORITY, Ap.TABLE_NAME, AP);
		matcher.addURI(AUTHORITY, Ap.TABLE_NAME+"/#", AP+ID);
		matcher.addURI(AUTHORITY, Ap.TABLE_NAME+"/bybssid", AP+AP);


		matcher.addURI(AUTHORITY, Ap.TABLE_NAME+Node.TABLE_NAME, NODE+AP);
		matcher.addURI(AUTHORITY, Ap.TABLE_NAME+"/#/"+Node.TABLE_NAME+"/#", NODE+ID+AP+ID);
		matcher.addURI(AUTHORITY, Ap.TABLE_NAME+Node.TABLE_NAME+"/#", NODE+AP+ID);

		//		matcher.addURI(AUTHORITY,"/wifi",WIFI);

		return matcher;
	}

	/* (non-Javadoc)
	 * @see android.content.ContentProvider#onCreate()
	 */
	@Override
	public boolean onCreate()
	{
		ArrayList<SQLEntety> enteties=new ArrayList<AbstractEntety.SQLEntety>();
		enteties.add(bEnt);
		enteties.add(fEnt);
		enteties.add(rEnt);
		enteties.add(aEnt);
		enteties.add(nEnt);

		//Send link to entities to access their createTable function defined in AbstractEntety.SQLEntety
		mOpenHelper = new DatabaseHelper(getContext(),enteties);
		return true;
	}

	/* (non-Javadoc)
	 * @see android.content.ContentProvider#query(android.net.Uri, java.lang.String[], java.lang.String, java.lang.String[], java.lang.String)
	 */
	@Override
	public Cursor query(Uri uri, String[] projection, String selection, String[] selectionArgs, String sortOrder)
	{
		Cursor c=null;
		int parent=0,parent2=0;
		SQLiteDatabase db = mOpenHelper.getReadableDatabase();
		switch (sURIMatcher.match(uri))
		{
			case BUILDING+ID: //TODO Get vertex of building
				break;
			case FLOOR+ID: //TODO Get vertex of floor
				break;
			case ROOM+ID: //TODO Get vertex of room
				break;
			case BUILDING: //Get all buildings
				try {c=db.rawQuery(bEnt.getAll(), null);} catch (Exception e)
				{IndMap.log(IndMap.ERROR, TAG, "Getting Buildings "+e.toString());}
				break;
			case FLOOR+BUILDING+ID: //Get all floors given building_id
				try{c=db.rawQuery(fEnt.getByParent((parent=Integer.parseInt(uri.getPathSegments().get(1)))), null);}
				catch (Exception e){IndMap.log(IndMap.ERROR, TAG, "Getting Floor by building #"+parent+" "+e.toString());}
				break;
			case ROOM+FLOOR+ID: //Get all rooms given floor_id
				try{c=db.rawQuery(rEnt.getByParent((parent=Integer.parseInt(uri.getPathSegments().get(1)))), null);}
				catch (Exception e){IndMap.log(IndMap.ERROR, TAG, "Getting Room by floor #"+parent+" "+e.toString());}
				break;

			case NODE:
				try {c=db.rawQuery(nEnt.getAll(),null);} catch (Exception e)
				{IndMap.log(IndMap.ERROR, TAG, "Getting Nodes"+e.toString());}
				break;
			case NODE+ID:
				try {c=db.rawQuery(nEnt.getOne((parent=Integer.parseInt(uri.getPathSegments().get(1)))), null);} catch (Exception e)
				{IndMap.log(IndMap.ERROR, TAG, "Getting Node #"+parent+" "+e.toString());}
				break;

			case NODE+NODE:
				try {c=db.rawQuery("SELECT * FROM "+Node.TABLE_NAME+Node.TABLE_NAME, null);} catch (Exception e)
				{IndMap.log(IndMap.ERROR, TAG, "Getting All node-nodes "+e.toString());}
				break;
			case NODE+NODE+ID:
				try {c=db.rawQuery(nEnt.getByParent((parent=Integer.parseInt(uri.getPathSegments().get(1)))), null);} catch (Exception e)
				{IndMap.log(IndMap.ERROR, TAG, "Getting Nodes by Node #"+parent+" "+e.toString());}				
				break;				

			case NODE+ID+NODE+ID:
				try {c=db.rawQuery(nEnt.getCost(
						(parent=Integer.parseInt(uri.getPathSegments().get(1))), 
						(parent2=Integer.parseInt(uri.getPathSegments().get(3)))
				), null);} catch (Exception e)
				{IndMap.log(IndMap.ERROR, TAG, "Getting Nodes Cost by Nodes #"+parent+" and "+parent2+" "+e.toString());}				
				break;


			case BUILDING-NODE:
				try {
					String sql=nEnt.getByBuilding(0);
					//					IndMap.log(IndMap.DEBUG, TAG, sql);
					c=db.rawQuery(sql, null);} catch (Exception e)
					{IndMap.log(IndMap.ERROR, TAG, "Getting outdoor Node "+e.toString());}				
					break;
			case NODE+BUILDING+ID:
				try {
					String sql=nEnt.getByBuilding((parent=Integer.parseInt(uri.getPathSegments().get(1))));
					//					IndMap.log(IndMap.DEBUG, TAG, sql);	
					c=db.rawQuery(sql, null);} catch (Exception e)
					{IndMap.log(IndMap.ERROR, TAG, "Getting Node by building #"+parent+" "+e.toString());}
					break;				
			case NODE+FLOOR+ID:
				try {
					String sql=nEnt.getByFloor((parent=Integer.parseInt(uri.getPathSegments().get(1))));
					//					IndMap.log(IndMap.DEBUG, TAG, sql);
					c=db.rawQuery(sql, null);} catch (Exception e)
					{IndMap.log(IndMap.ERROR, TAG, "Getting Node by floor #"+parent+" "+e.toString());}
					break;
			case NODE+ROOM+ID:
				try {
					String sql=nEnt.getByRoom((parent=Integer.parseInt(uri.getPathSegments().get(1))));
					//					IndMap.log(IndMap.DEBUG, TAG, sql);
					c=db.rawQuery(sql, null);} catch (Exception e)
					{IndMap.log(IndMap.ERROR, TAG, "Getting Node by room #"+parent+" "+e.toString());}
					break;

			case AP: //Get all AP's
				try {c=db.rawQuery(aEnt.getAll(), null);} catch (Exception e)
				{IndMap.log(IndMap.ERROR, TAG, "Getting AP's "+e.toString());}
				break;
			case AP+NODE:
				try {c=db.rawQuery("SELECT * FROM "+Ap.TABLE_NAME+Node.TABLE_NAME, null);} catch (Exception e)
				{IndMap.log(IndMap.ERROR, TAG, "Getting All ap-nodes "+e.toString());}
				break;				
			case AP+AP: //Get all AP's ordered by BSSID
				try {c=db.rawQuery(aEnt.getOne(0), null);} catch (Exception e)
				{IndMap.log(IndMap.ERROR, TAG, "Getting AP's ordered by BSSID"+e.toString());}
				break;
			case NODE+AP+ID:
				try {c=db.rawQuery(aEnt.getByParent((parent=Integer.parseInt(uri.getPathSegments().get(1)))),null);} catch (Exception e)
				{IndMap.log(IndMap.ERROR, TAG, "Getting Ap's by Node "+parent+" "+e.toString());}				
				break;
			default:
				throw new IllegalArgumentException("Unknown URI " + uri);
		}
		return c;
	}

	/* (non-Javadoc)
	 * @see android.content.ContentProvider#insert(android.net.Uri, android.content.ContentValues)
	 */
	@Override
	public Uri insert(Uri uri, ContentValues initValues)
	{
		String table=null;
		String id=null;
		long rowId=0;
		SQLiteDatabase db = mOpenHelper.getWritableDatabase();
		ContentValues values;
		if(initValues==null)values=new ContentValues();
		else values=new ContentValues(initValues);

		switch (sURIMatcher.match(uri))
		{
			case BUILDING+ID:
				id=uri.getPathSegments().get(1);
				if(!values.containsKey(Building._ID))values.put(Building._ID, Integer.parseInt(id));
			case BUILDING:
				table = Building.TABLE_NAME;
				break;

			case BUILDING+ID+FLOOR+ID:
				if(!values.containsKey(Floor._ID))values.put(Floor._ID, Integer.parseInt(uri.getPathSegments().get(3)));
			case BUILDING+FLOOR+ID:
				if(id==null)id=uri.getPathSegments().get(1);
				if(!values.containsKey(Floor._ID)) throw new SQLException("Failed to insert row into " + uri);
				if(!values.containsKey(Building.PARENT))values.put(Building.PARENT, Integer.parseInt(id));
				id=values.getAsString(Floor._ID);
			case FLOOR+ID:
				if(id==null)id=uri.getPathSegments().get(1);
				if(!values.containsKey(Floor._ID))values.put(Floor._ID, Integer.parseInt(id));
			case FLOOR:
				table = Floor.TABLE_NAME;
				break;

			case FLOOR+ID+ROOM+ID: //link floor and room on insert of room
				ContentValues val=new ContentValues();
				val.put(Floor.PARENT, Integer.parseInt(uri.getPathSegments().get(1)));
				val.put(Room.PARENT, Integer.parseInt(uri.getPathSegments().get(3)));
				db.insert(Floor.TABLE_NAME+Room.TABLE_NAME, null, val); //link floor and room on insert of room
				id=uri.getPathSegments().get(3);
			case ROOM+ID: //Do not use if not know to insert info floorroom also manually
				if(id==null)id=uri.getPathSegments().get(1);
				if(!values.containsKey(Room._ID))values.put(Room._ID, Integer.parseInt(id));
			case ROOM:
				table = Room.TABLE_NAME;
				break;

			case NODE+FLOOR+ID+ROOM+ID:
				id=uri.getPathSegments().get(3);
				if(!values.containsKey(Floor.PARENT))values.put(Floor.PARENT, Integer.parseInt(uri.getPathSegments().get(1)));
			case NODE+ROOM+ID:
				if(id==null)id=uri.getPathSegments().get(1);
				if(!values.containsKey(Room.PARENT))values.put(Room.PARENT, Integer.parseInt(id));

			case NODE+ID+NODE+ID:
				if(!values.containsKey(Node.PARENTA))values.put(Node.PARENTA, Integer.parseInt(uri.getPathSegments().get(1)));
				if(!values.containsKey(Node.PARENTB))values.put(Node.PARENTB, Integer.parseInt(uri.getPathSegments().get(3)));
			case NODE+NODE:
				table = Node.TABLE_NAME+Node.TABLE_NAME;
				break;

			case NODE+ID:
				if(!values.containsKey(AbstractEntety._ID))values.put(AbstractEntety._ID, Integer.parseInt(uri.getPathSegments().get(1)));
			case NODE: //Create new node, return id in uri
				if(!values.containsKey(AbstractEntety._ID))values.putNull(AbstractEntety._ID);
				table = Node.TABLE_NAME;
				break;

			case NODE+ID+AP+ID:
				if(!values.containsKey(Node.PARENT))values.put(Node.PARENT, Integer.parseInt(uri.getPathSegments().get(1)));
				if(!values.containsKey(Ap.PARENT))values.put(Ap.PARENT, Integer.parseInt(uri.getPathSegments().get(3)));				
			case NODE+AP:
				table = Ap.TABLE_NAME+Node.TABLE_NAME;
				break;
			case AP:
				table = Ap.TABLE_NAME;
				break;

			default:
				throw new IllegalArgumentException("Unknown URI " + uri);
		}
		rowId=db.insert(table, null, values);	
		if (rowId > 0)
		{
			Uri noteUri = Uri.withAppendedPath(CONTENT_URI, table+"/"+rowId);
			getContext().getContentResolver().notifyChange(noteUri, null);
			return noteUri;
		}
		throw new SQLException("Failed to insert row into " + uri);
	}

	/* (non-Javadoc)
	 * @see android.content.ContentProvider#delete(android.net.Uri, java.lang.String, java.lang.String[])
	 */
	@Override
	public int delete(Uri uri, String selection, String[] selectionArgs)
	{
		SQLiteDatabase db = mOpenHelper.getWritableDatabase();
		int n=0;
		String id,id2;
		switch (sURIMatcher.match(uri))
		{
			case BUILDING://Delete all buildings
				n += db.delete(Building.TABLE_NAME, null, null);	
			case FLOOR:   //Delete all floors, also if all buildings deleted
				n += db.delete(Floor.TABLE_NAME, null, null);
			case ROOM:    //Delete all rooms, also if all floors deleted
				n += db.delete(Room.TABLE_NAME, null, null);
				n += db.delete(Floor.TABLE_NAME+Room.TABLE_NAME, null, null);
				break;

			case AP:
				n += db.delete(Ap.TABLE_NAME, null, null);
				n += db.delete(Ap.TABLE_NAME+Node.TABLE_NAME, null, null);
				break;
			case NODE:
				n += db.delete(Node.TABLE_NAME, null, null);
				n += db.delete(Node.TABLE_NAME+Node.TABLE_NAME, null, null);
				break;
			case NODE+ID:
				id=uri.getPathSegments().get(1);
				n+=db.delete(Node.TABLE_NAME, Node._ID+"="+id,null);
				n+=db.delete(Node.TABLE_NAME+Node.TABLE_NAME, Node.PARENTA+"="+id+" OR "+Node.PARENTB+"="+id,null);
				break;

			case NODE+NODE+ID:
				id=uri.getPathSegments().get(1);
				n+=db.delete(Node.TABLE_NAME+Node.TABLE_NAME, Node.PARENTA+"="+id+" OR "+Node.PARENTB+"="+id,null);
				break;
				
			case NODE+ID+NODE+ID:
				id=uri.getPathSegments().get(1);
				id2=uri.getPathSegments().get(3);
				n+=db.delete(Node.TABLE_NAME+Node.TABLE_NAME, Node.PARENTA+"="+id+" AND "+Node.PARENTB+"="+id2,null);
				n+=db.delete(Node.TABLE_NAME+Node.TABLE_NAME, Node.PARENTA+"="+id2+" AND "+Node.PARENTB+"="+id,null);
				break;

			case AP+NODE+ID:
				n+=db.delete(Ap.TABLE_NAME+Node.TABLE_NAME, Node.PARENT+"="+uri.getPathSegments().get(1),null);
				break;

			case AP+ID:
				id=uri.getPathSegments().get(1);
				n+=db.delete(Ap.TABLE_NAME, Ap._ID+"="+id,null);
				n+=db.delete(Ap.TABLE_NAME+Node.TABLE_NAME, Node.PARENT+"="+id, null);
				break;
				/*
		case BUILDING+ID: //Delete one building
			n+= db.delete(,,);
		case FLOOR+BUILDING+ID: //Delete floors by building id
		    n+= db.delete(,,);
		case ROOM+BUILDING+ID: //Delete rooms by building id
		    n+= db.delete(,,);
		    break;
				 */
				/*
		case FLOOR+ID: //Delete one floor
			n+= db.delete(,,);
		case ROOM+FLOOR+ID: //Delete rooms by floor id
		    n+= db.delete(,,);
		    break;
				 */
			default:
				throw new SQLException("Failed to delete row " + uri);
		}
		getContext().getContentResolver().notifyChange(uri,null);
		return n;
	}

	/* (non-Javadoc)
	 * @see android.content.ContentProvider#getType(android.net.Uri)
	 */
	@Override
	public String getType(Uri uri)
	{
		return null;
	}

	/* (non-Javadoc)
	 * @see android.content.ContentProvider#update(android.net.Uri, android.content.ContentValues, java.lang.String, java.lang.String[])
	 */
	@Override
	public int update(Uri uri, ContentValues values, String selection, String[] whereArgs)
	{
		SQLiteDatabase db = mOpenHelper.getWritableDatabase();
		int n=0;
		switch (sURIMatcher.match(uri))
		{
			case NODE+ID+NODE+ID:
				String id1=uri.getPathSegments().get(1),
				id2=uri.getPathSegments().get(3);
				try{
					n+=db.update(Node.TABLE_NAME+Node.TABLE_NAME, values, 
							Node.PARENTA+"="+id1+" AND "+Node.PARENTB+"="+id2,
							null);
				}catch (Exception e) {}
				try{
					n+=db.update(Node.TABLE_NAME+Node.TABLE_NAME, values, 
							Node.PARENTB+"="+id1+" AND "+Node.PARENTA+"="+id2,
							null);
				}catch (Exception e) {}
				break;
			case NODE+ID:
				if(!values.containsKey(Node._ID))values.put(Node._ID, Integer.parseInt(uri.getPathSegments().get(1)));
			case NODE:
				if(!values.containsKey(Node._ID)) throw new SQLException("Failed to insert row into " + uri);
				n=db.update(Node.TABLE_NAME, values, Node._ID+"="+values.getAsString(Node._ID), whereArgs);
				break;
			default:
				throw new SQLException("Failed to update row " + uri);
		}
		getContext().getContentResolver().notifyChange(uri,null);
		return n;
	}
}
