/*********************************************************************************************
 * File:    Room.java
 * Project: Indoor maps, Masterthesis
 * Version: 0.5b
 * Author:  Jon Villy Meidell
 * Org:     NTNU
 * 
 * 
 * Data Structure
 * A Room
 */
package no.ntnu.map.indoormaps;

import java.security.InvalidParameterException;

import javax.microedition.khronos.opengles.GL10;

import no.ntnu.map.indoormaps.World.Colour;
import no.ntnu.map.indoormaps.Storage.MysqlViaPHP;
import no.ntnu.map.indoormaps.Storage.SQLStorage.QueryBuilder;
import no.ntnu.map.indoormaps.Storage.SQLStorage.QueryBuilder.WrongTypeException;

import org.json.JSONObject;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.graphics.Color;
import android.net.Uri;

// TODO: Auto-generated Javadoc
/**
 * The Class Room.
 */
public class Room extends AbstractEntety
{

	/** TAG used with {@link IndMap#log(short, String, String)}. */
	public static final String TAG="Room";

	/** The Database table name. Used in {@link Provider} and calls to it, {@link QueryBuilder}, ect*/
	public static final String TABLE_NAME = "room";

	/** The parent key as foreign key. */
	public static final String PARENT="room_id";

	/**
	 * The Class Entety.
	 */
	public static class Entety implements ShapeSQLEntety
	{

		/** The Constant SPINNER_TEXT_ALL. */
		public static final String[] PROJ_ALL={_ID,ID,NAME,WALL_BUF,FLOOR_BUF};
		/** The Constant SPINNER_TEXT_ALL. */
		public static final String[]	SPINNER_TEXT_ALL={"0","-","All","",""};

		/* (non-Javadoc)
		 * @see no.ntnu.map.indoormaps.AbstractEntety.SQLEntety#createTable()
		 */
		public String createTable() throws WrongTypeException
		{
			//Room table
			QueryBuilder qb=new QueryBuilder(QueryBuilder.CREATE);
			qb.setTable(TABLE_NAME);
			qb.addField(_ID,    "INTEGER", true);
			qb.addField(ID,     "TEXT");
			qb.addField(NAME,   "TEXT");
			qb.addField(WALL_BUF,	"BLOB");
			qb.addField(FLOOR_BUF,	"BLOB");
			String ret=qb.getSqlString();

			//FloorRoom table
			qb=new QueryBuilder(QueryBuilder.CREATE);
			qb.setTable(Floor.TABLE_NAME+TABLE_NAME);
			qb.addField(Floor.PARENT,"INTEGER",true);
			qb.addField(Room.PARENT,"INTEGER",true);
			ret+=qb.getSqlString();
			return ret;
		}

		/* (non-Javadoc)
		 * @see no.ntnu.map.indoormaps.AbstractEntety.SQLEntety#getAll()
		 */
		public String getAll() throws WrongTypeException
		{
			//Generate SQL-query to locate all rooms given the parent floor
			QueryBuilder sql=new QueryBuilder(QueryBuilder.SELECT);
			sql.addTable(TABLE_NAME);
			sql.addProjection(PROJ_ALL);
			return sql.getSqlString();
		}

		/* (non-Javadoc)
		 * @see no.ntnu.map.indoormaps.AbstractEntety.SQLEntety#getByParent(no.ntnu.map.indoormaps.AbstractEntety)
		 */
		public String getByParent(AbstractEntety parent) throws WrongTypeException
		{
			if(!(parent instanceof Floor))throw new InvalidParameterException("A Room's parent is supposed to be a Floor");
			return getByParent(parent.getI());
		}

		/* (non-Javadoc)
		 * @see no.ntnu.map.indoormaps.AbstractEntety.SQLEntety#getByParent(int)
		 */
		public String getByParent(int parent) throws WrongTypeException
		{
			//Generate SQL-query to locate all rooms given the parent floor
			QueryBuilder qb=new QueryBuilder(QueryBuilder.SELECT);
			qb.addTable(TABLE_NAME);
			qb.addTable(Floor.TABLE_NAME+TABLE_NAME);
			qb.addProjection(PROJ_ALL);
			qb.addJoinCond(PARENT, QueryBuilder.EQUALS, Room._ID);
			qb.addWhereCond(Floor.PARENT, QueryBuilder.EQUALS, parent);
			qb.addOrdering(ID);
			return qb.getSqlString();
		}

		/* (non-Javadoc)
		 * @see no.ntnu.map.indoormaps.AbstractEntety.SQLEntety#dropTable()
		 */
		public String dropTable() throws WrongTypeException
		{
			return SQLEntety.DROPTABLESQL+TABLE_NAME;
		}

		/* (non-Javadoc)
		 * @see no.ntnu.map.indoormaps.AbstractEntety.SQLEntety#getShapes(no.ntnu.map.indoormaps.AbstractEntety, short)
		 */
		public String getShapes(AbstractEntety obj, short match) throws WrongTypeException,InvalidParameterException
		{
			if(obj instanceof Room)
			{
				QueryBuilder qb=new QueryBuilder(QueryBuilder.SELECT);
				String table=Room.TABLE_NAME+Shape.TABLE_NAME;
				qb.addTable(table);
				Shape.join2ParentAndPoint(qb,table);
				qb.addWhereCond(Room.PARENT, QueryBuilder.EQUALS, obj.getI());
				switch(match)
				{
					case Shape.WALL:
						qb.addWhereCond(Shape.TYPE, QueryBuilder.NOT_EQUAL, Shape.TYPE_BASE);
						break;
					case Shape.BASE:
						qb.addWhereCond(Shape.TYPE, QueryBuilder.EQUALS, Shape.TYPE_BASE);
						break;
				}
				return qb.getSqlString();
			}
			else throw new InvalidParameterException("Input is not a Room");
		}

		/* (non-Javadoc)
		 * @see no.ntnu.map.indoormaps.AbstractEntety.SQLEntety#getOne(int)
		 */
		public String getOne(int id) throws WrongTypeException
		{
			//Generate SQL-query to locate all rooms given the parent floor
			QueryBuilder qb=new QueryBuilder(QueryBuilder.SELECT);
			qb.addTable(TABLE_NAME);
			qb.addProjection(PROJ_ALL);
			qb.addWhereCond(_ID, QueryBuilder.EQUALS, id);
			return qb.getSqlString();
		}
	}

	/* (non-Javadoc)
	 * @see no.ntnu.map.indoormaps.AbstractEntety#toInternal(android.content.Context)
	 */
	@Override
	void toInternal(Context c, int i)
	{
		ContentValues values=new ContentValues();
		values.put(_ID, _id);
		values.put(ID,id);
		values.put(NAME,name);
		values.put(WALL_BUF, int2Byte(vWall));
		values.put(FLOOR_BUF, int2Byte(vFloor));
		try{c.getContentResolver().insert(Uri.withAppendedPath(Provider.CONTENT_URI, Floor.TABLE_NAME+"/"+floor.getI()+"/"+TABLE_NAME+"/"+_id), values);}catch (Exception e) {
			IndMap.log(IndMap.ERROR, TAG, "Error save to internal "+e.toString());
		}
	}

	/**
	 * The building.
	 * @uml.property  name="building"
	 * @uml.associationEnd  multiplicity="(1 1)" inverse="rooms:no.ntnu.map.indoormaps.Building"
	 */
	private Building building;

	/**
	 * The floor.
	 * @uml.property  name="floor"
	 * @uml.associationEnd  multiplicity="(1 1)" inverse="rooms:no.ntnu.map.indoormaps.Floor"
	 */
	private Floor floor;

	/**
	 * The node.
	 * @uml.property  name="node"
	 * @uml.associationEnd  inverse="room:no.ntnu.map.indoormaps.Node"
	 */
	private Node node;

	/**
	 * Instantiates a new room.
	 *
	 * @param c A {@link Cursor} giving buildings from {@link Provider internal storage} 
	 * @param parent the parent
	 * @param gparent the gparent
	 */
	Room(Cursor c, Floor parent, Building gparent)
	{
		building=gparent;
		floor=parent;
		_id=c.getInt(c.getColumnIndex(_ID));
		id=c.getString(c.getColumnIndex(ID));
		name=c.getString(c.getColumnIndex(NAME));
		vFloor=byte2Int(c.getBlob(c.getColumnIndex(FLOOR_BUF)));
		vWall=byte2Int(c.getBlob(c.getColumnIndex(WALL_BUF)));
		if(IndMap.LOAD_INT_DEBUG)IndMap.log(IndMap.DEBUG, TAG, "In.New "+toString());
	}

	/**
	 * Gets the building.
	 * @return  the building
	 * @uml.property  name="building"
	 */
	public Building getBuilding(){return building;}

	/**
	 * Sets the building.
	 * @param building  the new building
	 * @uml.property  name="building"
	 */
	public void setBuilding(Building building){this.building = building;}

	/**
	 * Gets the floor.
	 * @return  the floor
	 * @uml.property  name="floor"
	 */
	public Floor getFloor(){return floor;}

	/**
	 * Sets the floor.
	 * @param floor  the new floor
	 * @uml.property  name="floor"
	 */
	public void setFloor(Floor floor){this.floor = floor;}

	/**
	 * Gets the node.
	 * @return  the node
	 * @uml.property  name="node"
	 */
	public Node getNode(){return node;}

	/**
	 * Sets the node.
	 * @param node  the new node
	 * @uml.property  name="node"
	 */
	public void setNode(Node node)
	{
		this.node = node;
		if(node.getRoom().equals(this))return;
		node.setRoom(this);
	}

	/**
	 * Instantiates a new room.
	 *
	 * @param json the json
	 * @param parent the parent
	 * @param gparent the gparent
	 */
	Room(JSONObject json, Floor parent, Building gparent) 
	{
		try
		{
			//Generate itself from the result of a sql-query
			building=gparent;
			floor=parent;
			_id=json.getInt(_ID);
			id=json.getString(ID);
			name=json.getString(NAME);
			if(IndMap.LOAD_EX_DEBUG)IndMap.log(IndMap.DEBUG, TAG, "Ex.New "+toString());
		}
		catch (Exception e) {
			IndMap.log(IndMap.ERROR, TAG, "Error conv from JSON to this "+TAG+""+e.toString());
			return;
		}
	}

	/**
	 * Load from database.
	 *
	 * @param s the s
	 * @param c the c
	 * @param level the level
	 */
	void loadFromDatabase(MysqlViaPHP s,Context c,int level)
	{
		ShapeSQLEntety rSQL=new Room.Entety();
		try
		{
			s.executeQuery(rSQL.getShapes(this, Shape.BASE));
			floorFromDatabase(s,level);
			s.executeQuery(rSQL.getShapes(this, Shape.WALL));
			wallFromDatabase(s,level);

			toInternal(c,0);
			generateBuffers();
		}
		catch (Exception e) {
			IndMap.log(IndMap.ERROR, TAG, "Get Vertexes "+e.toString());
			return;
		}
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString()
	{
		return id+" - "+name;
	}

	/* (non-Javadoc)
	 * @see no.ntnu.map.indoormaps.AbstractEntety#draw(javax.microedition.khronos.opengles.GL10, no.ntnu.map.indoormaps.World)
	 */
	@Override
	public void draw(GL10 gl, World w)
	{
		Colour c=World.COL_NULL_AT_WALL;
		Room at=w.getAtRoom();
		if(equals(at))c=World.COL_AT_WALL;
		else if(at!=null)c=World.COL_NOT_AT_WALL;
		
		draw(gl, GL10.GL_TRIANGLES, vWallBuffer, num_vWall, Color.argb(c.alpha*(w.isMode_draw2D()?1/2:1), c.red, c.green, c.blue));

		c=new Colour(255, 160, 0, 100);
		//IndMap.log(IndMap.DEBUG, TAG, "Drawing Building num floor vertex: "+num_vFloor);
		draw(gl, GL10.GL_TRIANGLES, vFloorBuffer, num_vFloor, Color.argb(c.alpha, c.red, c.green, c.blue));
	}
}
