/*********************************************************************************************
 * Project: Indoor maps, Masterthesis
 * Version: 1.0b
 * Author:  Jon Villy Meidell
 * Org:     NTNU
 */
package no.ntnu.map.indoormaps;

import java.security.InvalidParameterException;
import java.util.ArrayList;
import java.util.List;

import javax.microedition.khronos.opengles.GL10;

import no.ntnu.map.indoormaps.World.Colour;
import no.ntnu.map.indoormaps.Storage.SQLStorage.QueryBuilder.WrongTypeException;

import android.content.Context;
import android.graphics.Color;

// TODO: Auto-generated Javadoc
/**
 * The Class Route.
 */
public class Route extends AbstractEntety
{

	/** The Constant TAG. */
	public static final String TAG="Route";

	/** The COL_DEFAULT. */
	public static Colour DEFAULT=new Colour(50, 50, 255, 255);

	/** The HIG h_ light. */
	public static Colour HIGH_LIGHT=new Colour(255, 255, 255, 255);

	/**
	 * The Class Entety.
	 */
	public static class Entety implements SQLEntety
	{

		/* (non-Javadoc)
		 * @see no.ntnu.map.indoormaps.AbstractEntety.SQLEntety#createTable()
		 */
		public String createTable() throws WrongTypeException
		{
			// TODO
			return null;
		}

		/* (non-Javadoc)
		 * @see no.ntnu.map.indoormaps.AbstractEntety.SQLEntety#dropTable()
		 */
		public String dropTable() throws WrongTypeException
		{
			// TODO
			return null;
		}

		/* (non-Javadoc)
		 * @see no.ntnu.map.indoormaps.AbstractEntety.SQLEntety#getAll()
		 */
		public String getAll() throws WrongTypeException
		{
			// TODO
			return null;
		}

		/* (non-Javadoc)
		 * @see no.ntnu.map.indoormaps.AbstractEntety.SQLEntety#getByParent(no.ntnu.map.indoormaps.AbstractEntety)
		 */
		public String getByParent(AbstractEntety parent) throws WrongTypeException{return getByParent(0);}

		/* (non-Javadoc)
		 * @see no.ntnu.map.indoormaps.AbstractEntety.SQLEntety#getByParent(int)
		 */
		public String getByParent(int parent_id) throws WrongTypeException,InvalidParameterException {throw new InvalidParameterException("A building does not have a parent");}

		/* (non-Javadoc)
		 * @see no.ntnu.map.indoormaps.AbstractEntety.SQLEntety#getShapes(no.ntnu.map.indoormaps.AbstractEntety, short)
		 */
		/**
		 * Gets the shapes.
		 *
		 * @param obj the obj
		 * @param match the match
		 * @return the shapes
		 * @throws WrongTypeException the wrong type exception
		 */
		public String getShapes(AbstractEntety obj, short match) throws WrongTypeException
		{
			return null;
		}

		/* (non-Javadoc)
		 * @see no.ntnu.map.indoormaps.AbstractEntety.SQLEntety#getOne(int)
		 */
		public String getOne(int id) throws WrongTypeException
		{
			return null;
		}
	}


	/**
	 * The nodes.
	 * @uml.property  name="nodes"
	 * @uml.associationEnd  multiplicity="(0 -1)" elementType="no.ntnu.map.indoormaps.Node"
	 */
	private ArrayList<Node> nodes;

	/**
	 * The targets.
	 * @uml.property  name="targets"
	 */
	private ArrayList<Node> targets;

	/** The pos. */
	private int pos=0;

	/** The siderot. */
	private float siderot=0.0f;
	/** The uprot. */
	private float uprot=0.0f;

	/**
	 * Gets the siderot.
	 *
	 * @return the siderot
	 */
	public float getSiderot()
	{
		return siderot;
	}

	/**
	 * Gets the uprot.
	 *
	 * @return the uprot
	 */
	public float getUprot()
	{
		return uprot;
	}

	/**
	 * Instantiates a new route.
	 *
	 * @param targets the targets
	 */
	public Route(List<Node> targets)
	{
		nodes=new ArrayList<Node>(targets);
		this.targets=new ArrayList<Node>(targets);
		makeGraphics();
	}

	/**
	 * Instantiates a new route.
	 *
	 * @param known the known
	 * @param ids the ids
	 * @param genRoute the gen route
	 */
	public Route(Node[] known, int[] ids, boolean genRoute)
	{
		targets=new ArrayList<Node>(ids.length);
		for(int i:ids)
		{
			Node n=(Node) Node.searchById(known, i);
			if(n!=null)targets.add(n);
		}
		if(genRoute)generateRoute();
		else nodes=targets;
		makeGraphics();
	}
	
	/**
	 * Goto this.
	 *
	 * @param to the to
	 * @return true, if successful
	 */
	public boolean gotoThis(Node to)
	{
		int i=-1;
		boolean b=(to!=null&&(i=nodes.indexOf(to))>-1);
		if(b)
		{
			pos=i;
			if(pos+1<nodes.size())
			{
				calcAngle(getNow().getPoint(), nodes.get(pos+1).getPoint());
			}
			else gotoEnd();
		}
		if(IndMap.MOVE_DEBUG)IndMap.log(IndMap.DEBUG, "MOVE", "** Route goto Node: "+(to==null?"Not Found":to.toString())+" in route? "+b);
		return b;
	}

	/**
	 * Goto start.
	 *
	 * @return the node
	 */
	public Node gotoStart()
	{
		if(IndMap.MOVE_DEBUG)IndMap.log(IndMap.DEBUG, "MOVE", "** Route goto First");
		pos=0;
		calcAngle(getNow().getPoint(), nodes.get(pos+1).getPoint());
		return getNow();
	}

	/**
	 * Goto end.
	 *
	 * @return the node
	 */
	public Node gotoEnd()
	{
		if(IndMap.MOVE_DEBUG)IndMap.log(IndMap.DEBUG, "MOVE", "** Route goto Last");
		pos=nodes.size()-1;
		calcAngle(getNow().getPoint(),nodes.get(pos-1).getPoint());
		return getNow();
	}

	/**
	 * Goto next.
	 *
	 * @return the node
	 */
	public Node gotoNext()
	{
		if(IndMap.MOVE_DEBUG)IndMap.log(IndMap.DEBUG, "MOVE", "** Route goto Next");
		pos++;
		if(pos+1<nodes.size())
		{
			calcAngle(getNow().getPoint(), nodes.get(pos+1).getPoint());
			return getNow();
		}
		else return gotoEnd();
	}

	/**
	 * Goto prev.
	 *
	 * @return the node
	 */
	public Node gotoPrev()
	{
		if(IndMap.MOVE_DEBUG)IndMap.log(IndMap.DEBUG, "MOVE", "** Route goto Prevous");
		pos--;
		if(pos>0)
		{
			calcAngle(getNow().getPoint(), nodes.get(pos+1).getPoint());
			return getNow();
		}
		else return gotoStart();
	}

	/**
	 * Gets the now.
	 *
	 * @return the now
	 */
	public Node getNow(){return nodes.get(pos);}

	/**
	 * Gets the last.
	 *
	 * @return the last
	 */
	public Node getLast(){return nodes.get(nodes.size()-1);}

	/**
	 * Calc angle.
	 *
	 * @param a the a
	 * @param c the c
	 */
	private void calcAngle(Point a, Point c)
	{

		// xy-plane, z-rotation
		siderot=Point.calcAngle(a.x, a.y, c.x, c.y);
//		if(!Float.isNaN(f))siderot=f;
//		else IndMap.log(IndMap.ERROR,TAG,"calcAngle siderot got NaN");
		

		// relative xz-plane, y-rotation
		uprot=0.0f;
		if(a.z==c.z)uprot=0.0f;
		else
		{
			double abdx=c.x-a.x,abdy=c.y-a.y;
			float xy_axis=(float)Math.sqrt(abdx*abdx+abdy*abdy);
			uprot=90.0f - Point.calcAngle(a.z, 0.0f,c.z, xy_axis);
//			if(!Float.isNaN(f))uprot=f;
//			else IndMap.log(IndMap.ERROR,TAG,"calcAngle uprot got NaN");
		}
	}
	/**
	 * Checks if is member.
	 *
	 * @param n the n
	 * @return true, if is member
	 */
	public boolean isMember(Node n){return nodes.contains(n);}

	/**
	 * Generate route.
	 */
	public void generateRoute()
	{
		ArrayList<Node> ret=new ArrayList<Node>(targets.size()*3);
		for(int i=1;i<targets.size();i++)
		{
			ArrayList<Node> retloc=makeRoute(targets.get(i-1),targets.get(i));
			ret.addAll(retloc.subList(1,retloc.size()));
		}

		//TODO gjre ferdig makeRoute, s kan convertere resultatet til nodes arraylista.
		//nodes=ret;
		nodes=targets;
		makeGraphics();
	}

	/**
	 * Make route.
	 *
	 * @param start the start
	 * @param stop the stop
	 * @return the route
	 */
	public static ArrayList<Node> makeRoute(Node start, Node stop) //A* search between each node , makes the lists of nodes from and with start to and with stop.
	{
		//TODO A* implementering
		return null;
	}

	/* (non-Javadoc)
	 * @see no.ntnu.map.indoormaps.AbstractEntety#toInternal(android.content.Context)
	 */
	@Override
	public void toInternal(Context c, int i)
	{

	}

	/* (non-Javadoc)
	 * @see no.ntnu.map.indoormaps.AbstractEntety#draw(javax.microedition.khronos.opengles.GL10, no.ntnu.map.indoormaps.World)
	 */
	@Override
	public void draw(GL10 gl, World world)
	{
		Colour c=DEFAULT;
		gl.glPushMatrix();

		try{
			gl.glTranslatef(0.0f, Node.FROM_FLOOR, 0.0f);
			draw(gl, GL10.GL_LINE_STRIP, vFloorBuffer, num_vFloor, Color.argb(c.alpha, c.red, c.green, c.blue));
		}catch (Exception e) {
			IndMap.log(IndMap.ERROR, TAG, "Draw "+e.toString());
		}

		gl.glPopMatrix();
	}

	/**
	 * Make graphics.
	 */
	private void makeGraphics() //Should be called each time targets is changed
	{
		vFloor=new int[nodes.size()*3];
		int n=0;
		for(int i=0;i<nodes.size();i++)
		{
			//X = y' Y = z' Z = -x'
			Point p=nodes.get(i).getPoint();
			vFloor[n++]=toFixed(p.y);
			vFloor[n++]=toFixed(p.z);
			vFloor[n++]=toFixed(-p.x);
		}
		generateBuffers();
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString()
	{
		StringBuilder sb=new StringBuilder("Route: Has "+nodes.size()+" nodes");
		/*if(nodes.size()>0)
		{
			sb.append("Has nodes (id): (");
			for(int i=0;i<nodes.size();i++)sb.append(nodes.get(i).getI()+", ");
			sb.append(")");
		}*/
		return sb.toString();
	}
	protected static Route makeDummy(Node at, Node to)
	{
		ArrayList<Node> ne=new ArrayList<Node>(2);
		ne.add(at);
		ne.add(to);

		Route ret=new Route(ne); //Utnytte route til  f ferdigutregnet vinkel til neste node.
		ret.gotoStart();
		return ret;
	}
}
