/*********************************************************************************************
 * Project: Indoor maps, Masterthesis
 * Version: 1.0b
 * Author:  Jon Villy Meidell
 * Org:     NTNU
 */

package no.ntnu.map.indoormaps;

import no.ntnu.map.indoormaps.AbstractEntety.SQLEntety;
import no.ntnu.map.indoormaps.Node.NodeSQLEntety;
import no.ntnu.map.indoormaps.Point.RotatePoint;
import no.ntnu.map.indoormaps.Storage.MysqlViaPHP;
import no.ntnu.map.indoormaps.Storage.SQLStorage.QueryBuilder;

import android.content.Context;
import android.database.Cursor;
import android.graphics.Color;
import android.net.Uri;
import android.os.Handler;
import android.os.Message;
import android.widget.Toast;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.IntBuffer;
import java.util.ArrayList;
import java.util.Iterator;

import javax.microedition.khronos.opengles.GL10;

// TODO: Auto-generated Javadoc
/**
 * The Class World.
 */
public class World
{

	/** TAG used with {@link IndMap#log(short, String, String)}. */
	public static final String TAG="World";

	/** The Constant STEP_UP_ROT. */
	private static final float STEP_FWD=20.0f;

	/** The Constant STEP_UP_ROT. */
	private static final float	STEP_SIDE=15f;

	/** The Constant STEP_UP_ROT. */
	private static final float	STEP_SIDE_ROT=10.0f;

	/** The Constant STEP_UP_ROT. */
	private static final float	STEP_UP_ROT=8.0f;

	/** The FRO m_ floor. */
	public static final short FROM_FLOOR=55; //TODO justere plassering fra gulv

	/** The Constant MOVE_DOWN. */
	public static final int MOVE_UPD=0;

	/** The Constant MOVE_DOWN. */
	public static final int	MOVE_FWD=1;

	/** The Constant MOVE_DOWN. */
	public static final int	MOVE_BWD=2;

	/** The Constant MOVE_DOWN. */
	public static final int	MOVE_LEFT=3;

	/** The Constant MOVE_DOWN. */
	public static final int	MOVE_RIGHT=4;

	/** The Constant MOVE_DOWN. */
	public static final int	MOVE_ROT_LEFT=5;

	/** The Constant MOVE_DOWN. */
	public static final int	MOVE_ROT_RIGHT=6;

	/** The Constant MOVE_DOWN. */
	public static final int	MOVE_ROT_UP=7;

	/** The Constant MOVE_DOWN. */
	public static final int	MOVE_ROT_DOWN=8;

	/** The Constant MOVE_DOWN. */
	public static final int	MOVE_UP=9;

	/** The Constant MOVE_DOWN. */
	public static final int	MOVE_DOWN=10;

	/** The Constant TRANS_NOT_ROT. */
	public static final short TRANS_ALL=1,TRANS_NOT_ROT=2,TRANS_NOT_MOVE=3;
	/** The trans length. */
	private final short TRANS_LENGTH=128;

	private IntBuffer mode2Dpoint;

	public static final Colour COL_MODE_2D_AT_POINT=new Colour(0, 0, 0, 200);

	public static final Colour COL_AT_WALL		= new Colour(255, 200, 40, 255);
	public static final Colour COL_NOT_AT_WALL	= new Colour(0, 100, 50, 180);
	public static final Colour COL_NULL_AT_WALL = new Colour(0, 180, 80, 180);
	public static final Colour COL_AT_FLOOR		= new Colour(160, 0, 0, 255);
	public static final Colour COL_NOT_AT_FLOOR	= new Colour(120, 120, 120, 180);

	/**
	 * The Class Colour.
	 */
	public static class Colour
	{

		/** The alpha. */
		public int red;
		/** The alpha. */
		public int	green;
		/** The alpha. */
		public int	blue;
		/** The alpha. */
		public int	alpha;

		/**
		 * Instantiates a new colour.
		 *
		 * @param r the r
		 * @param g the g
		 * @param b the b
		 * @param a the a
		 */
		public Colour(int r,int g,int b,int a)
		{
			red=r;
			green=g;
			blue=b;
			alpha=a;
		}
	}

	/**
	 * Gets the lFE thread.
	 *
	 * @return the lFE thread
	 */
	protected class LFEThread extends Thread
	{

		/** The m handler. */
		private Handler mHandler;

		/** The total. */
		private int total=0;

		/** The msg. */
		private String basemsg="";

		/** The msg. */
		private String	msg="";

		/**
		 * The s.
		 */
		private MysqlViaPHP s;

		/** The c. */
		private Context c;

		/** The incr. */
		int incr=0;

		/**
		 * Instantiates a new lFE thread.
		 *
		 * @param statement the statement
		 * @param context the context
		 * @param h the h
		 */
		public LFEThread(MysqlViaPHP statement,Context context,Handler h)
		{
			mHandler = h;
			s=statement;
			c=context;
		}

		/* (non-Javadoc)
		 * @see java.lang.Thread#run()
		 */
		public void run()
		{
			ArrayList<Building> bs=new ArrayList<Building>(10);
			ArrayList<Room> r = new ArrayList<Room>(1000);		
			ArrayList<Ap> a=new ArrayList<Ap>(200);
			ArrayList<Node> n=new ArrayList<Node>(1500);

			loadState=true; //For disabling drawing while loading
			SQLEntety bSQL=new Building.Entety(),aSQL=new Ap.Entety();
			NodeSQLEntety nSQL=new Node.Entety();
			try
			{
				if(IndMap.LOAD_EX_DEBUG)IndMap.log(IndMap.DEBUG, TAG, "Start loading external");
				setBaseMessage("Clearing..");
				//Remove old Buildings, Floors and Rooms
				c.getContentResolver().delete(Uri.withAppendedPath(Provider.CONTENT_URI, Building.TABLE_NAME), null, null);
				c.getContentResolver().delete(Uri.withAppendedPath(Provider.CONTENT_URI, Ap.TABLE_NAME), null, null);
				c.getContentResolver().delete(Uri.withAppendedPath(Provider.CONTENT_URI, Node.TABLE_NAME), null, null);

				setBaseMessage("Loading buildings");
				setMessage("Querying database");

				//Gets the sql from the Building class itself
				s.executeQuery(bSQL.getAll());
				setMessage("Processing");
				//Adds all buildings
				while(s.next())
				{
					bs.add(new Building(s.getJSONObject()));
				}
				buildings=new Building[bs.size()];
				bs.toArray(buildings);
				setProgress(5);

				//Makes the buildings find their floors and rooms. Returns all rooms to set up nodes
				Iterator<Building> i=bs.iterator();
				if(bs.size()>0)
				{
					incr=(int) (60f/bs.size());
					while(i.hasNext())
					{
						Building b=i.next();
						setBaseMessage("Loading building: "+b.getName());
						r.addAll(b.loadFromDatabase(s,c,this));
					}
				}
				setProgress(65);

				//Add all known APs
				//Gets the sql from the AP class itself
				setBaseMessage("Loading AP's");
				setMessage("Querying database");
				s.executeQuery(aSQL.getOne(0));
				setMessage("Processing");
				while(s.next())
				{
					Ap ap=new Ap(s.getJSONObject());
					a.add(ap);
					ap.toInternal(c,0);
				}
				knownAPs=new Ap[a.size()];
				a.toArray(knownAPs);
				if(IndMap.LOAD_EX_DEBUG)IndMap.log(IndMap.DEBUG, TAG, "Got "+a.size()+" Ap's");

				//Adds all nodes
				setProgress(70);
				setBaseMessage("Loading Nodes");

				//Gets the sql from the Node class itself
				setMessage("Querying database");
				s.executeQuery(nSQL.getAll());
				setMessage("Processing");
				while(s.next())
				{
					Node no=new Node(s.getJSONObject());
					n.add(no);
				}
				nodes=new Node[n.size()];
				n.toArray(nodes);
				if(IndMap.LOAD_EX_DEBUG)IndMap.log(IndMap.DEBUG, TAG, "Got "+n.size()+" Nodes's");

				//Find nodes linked to a room, and link them
				setBaseMessage("Loading Relationships");
				Iterator<Room> j=r.iterator();
				if(r.size()>0)
				{
					incr=(int)(15f/r.size());
					while(j.hasNext())
					{
						Room parent=j.next();
						s.executeQuery(nSQL.getByRoom(parent.getI()));

						while(s.next())
						{
							Node child=(Node) Node.searchById(nodes, s.getJSONObject().getInt(Node._ID));
							if(child!=null)child.setRoom(parent);
						}
						if(IndMap.LOAD_EX_DEBUG)IndMap.log(IndMap.DEBUG, TAG, "Got "+s.getNumJSONObjects()+" nodes on room "+parent.toString());
						doIncr(incr);
					}
				}
				for(Building b:buildings)for(Floor f:b.getFloors())
				{
					s.executeQuery(nSQL.getByFloor(f.getI()));
					while(s.next())
					{
						Node child=(Node) Node.searchById(nodes, s.getJSONObject().getInt(Node._ID));
						if(child!=null)child.setFloor(f);
					}
				}

				setProgress(85);

				//Link AP and Node
				Iterator<Node> k=n.iterator();
				ArrayList<Integer> level;
				ArrayList<Ap> ap;
				if(n.size()>0)
				{
					incr=(int)(6f/r.size());
					while(k.hasNext())
					{
						Node parent=k.next();
						setMessage("Querying database");
						s.executeQuery(aSQL.getByParent(parent.getI()));
						setMessage("Processing");
						level= new ArrayList<Integer>();
						ap  = new ArrayList<Ap>();
						if(IndMap.LOAD_EX_DEBUG)IndMap.log(IndMap.DEBUG, TAG, "Got Node : "+parent.toString()+" Got "+s.getNumJSONObjects()+" AP's  -  Known AP's: "+knownAPs.length);
						while(s.next())
						{
							Ap child=(Ap) Ap.getApByBSSID(knownAPs, Ap.stringSSID2long(s.getJSONObject().getString(Ap.BSSID),Ap.BSSID_SEPERATOR));
							if(child!=null)
							{
								int lev=0;
								if(IndMap.LOAD_EX_DEBUG)IndMap.log(IndMap.DEBUG, TAG, " - Got Ap: "+child.toString());
								ap.add(child);
								lev=s.getJSONObject().getInt(Ap.LEVELatNODE);
								level.add(new Integer(lev));
								child.addNode(parent);
							}
							else if(IndMap.LOAD_EX_DEBUG)IndMap.log(IndMap.DEBUG, TAG, " - Got NULL");
						}
						int[] levels=new int[level.size()];
						for(int h=0;h<level.size();h++)levels[h]=level.get(h);
						Ap[] aps=new Ap[ap.size()];
						ap.toArray(aps);
						parent.setAPsandLevels(aps,levels);
						doIncr(incr);
					}
				}
				if(IndMap.LOAD_EX_DEBUG)IndMap.log(IndMap.DEBUG, TAG, "Linked Aps and Nodes");

				//Link nodes
				k=n.iterator();
				setMessage("Linking nodes");
				if(n.size()>0)
				{
					incr=(int)(9f/r.size());
					while(k.hasNext())
					{
						Node parent=k.next();
						s.executeQuery(nSQL.getByParent(parent.getI()));
						while(s.next())
						{
							Node child=(Node) Node.searchById(nodes, s.getJSONObject().getInt(Node._ID));
							if(child!=null)parent.addNeighbor(child, MysqlViaPHP.toFloat(s.getJSONObject().getDouble(Node.CHILD_COST)));
						}
						if(IndMap.LOAD_EX_DEBUG)IndMap.log(IndMap.DEBUG, TAG, "Got "+s.getNumJSONObjects()+" neightbours to node "+parent.toString());
						parent.generateBuffers();
						parent.sortNeighborsCalcAngles();
						doIncr(incr);
					}
				}

				//Saving nodes to internal (this late since have to find Room, neighbor nodes and Ap's)
				k=n.iterator();
				while(k.hasNext())k.next().toInternal(c, 0);

				if(IndMap.LOAD_EX_DEBUG)IndMap.log(IndMap.DEBUG, TAG, "Finished!");

				setProgress(100);
				loadState=false;
				try{Thread.sleep(500);} catch(InterruptedException e1){IndMap.log(IndMap.ERROR, TAG, "LFEThread - sleep "+e1.toString());}
				setProgress(110);
			}
			catch (Exception e) {
				IndMap.log(IndMap.ERROR, TAG, "LFEThread "+e.toString());
				setMessage("ERROR!");
				try{Thread.sleep(1500);} catch(InterruptedException e1){IndMap.log(IndMap.ERROR, TAG, "LFEThread - sleep "+e1.toString());}
				setProgress(110);
			}
		}

		/**
		 * Sets the base message.
		 *
		 * @param msg the new base message
		 */
		private void setBaseMessage(String msg){basemsg=msg;this.msg="";notifyHandler();}

		/**
		 * Sets the message.
		 *
		 * @param msg the new message
		 */
		protected void setMessage(String msg){this.msg=msg;notifyHandler();}

		/**
		 * Gets the incr.
		 * @return  the incr
		 */
		protected int getIncr(){return (incr<1?1:incr);}

		/**
		 * Do incr.
		 *
		 * @param n the n
		 */
		protected void doIncr(int n){setProgress(total+n);}

		/**
		 * Sets the progress.
		 *
		 * @param msg the new progress
		 */
		private void setProgress(int msg){total=msg;notifyHandler();}

		/**
		 * Notify handler.
		 */
		private void notifyHandler()
		{
			Message ms = mHandler.obtainMessage();
			ms.obj = (String ) basemsg+(msg!=null?" - "+msg:"NULL!")+"...";
			ms.arg1 = total;
			mHandler.sendMessage(ms);
		}
	}

	/**
	 * The scale.
	 */
	public float scale=1.0f;

	/**
	 * The known a ps.
	 */
	private Ap[] knownAPs=new Ap[0];

	/**
	 * The buildings.
	 */
	private Building[] buildings=new Building[0];

	/**
	 * The nodes.
	 */
	private Node[] nodes=new Node[0];

	//private ArrayList<Route> routes=new ArrayList<Route>(); //TODO Routes from database

	/**
	 * The camera.
	 */
	private RotatePoint camera=new RotatePoint(0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f);

	/** The newcamera. */
	private RotatePoint newcamera=camera;

	/** The trans count. */
	private short transCount=0;

	/** The trans2 d count. */
	private short trans2DCount=0;

	/** The trans goal. */
	private RotatePoint transGoal=new RotatePoint(0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f);

	/** The mode2D position. */
	private RotatePoint mode2d_pos=new RotatePoint(500f,200f,1000f,0.0f,-90f,-90f);

	/**
	 * The at building.
	 */
	private Building atBuilding=null;

	/**
	 * The at floor.
	 */
	private Floor atFloor=null;

	/**
	 * The at room.
	 */
	private Room atRoom=null;

	/**
	 * The at route.
	 */
	private Route atRoute=null;

	/**
	 * The at node.
	 */
	private Node atNode=null;

	private Node last=null;

	/**
	 * The at next node index.
	 */
	private int nextNodeIndex=-1;

	/**
	 * The load state.
	 */
	private boolean loadState  = false; //Deactivate drawing when loading

	/**
	 * The mode_draw2 d.
	 */
	private boolean mode_draw2D= false; //2D mode, shown from above 

	/**
	 * The mode_node.
	 */
	private boolean mode_node  = false; //Navigate by node to node

	public World()
	{
		int[] a=new int[]
		                {
				-5,0,-5,
				5,0,-5,
				-5,0, 5,
				5,0, 5,
				0,0, 8
		                };

		ByteBuffer bb = ByteBuffer.allocateDirect(a.length*4);
		bb.order(ByteOrder.nativeOrder());
		mode2Dpoint = bb.asIntBuffer();
		for(int i=0;i<a.length;i++)mode2Dpoint.put(AbstractEntety.toFixed(a[i]));
	}


	/**
	 * Gets the loadstate.
	 *
	 * @return the loadstate
	 */
	public boolean getLoadstate(){return loadState;}

	/**
	 * Sets the loadstate.
	 *
	 * @param b the new loadstate
	 */
	public void setLoadstate(boolean b){loadState=b;}

	/**
	 * Checks if is mode_node.
	 *
	 * @return true, if is mode_node
	 */
	public boolean isMode_node(){return mode_node;}

	/**
	 * Checks if is mode_draw2 d.
	 *
	 * @return true, if is mode_draw2 d
	 */
	public boolean isMode_draw2D(){return mode_draw2D;}

	/**
	 * Gets the nodes.
	 * @return  the nodes
	 */
	public Node[] getNodes(){return nodes;}

	/**
	 * Gets the aps.
	 *
	 * @return the aps
	 */
	public Ap[] getAps(){return knownAPs;}

	/**
	 * Gets the camera.
	 * @return  the camera
	 */
	public RotatePoint getCamera(){return newcamera;}

	/**
	 * Sets the camera.
	 * @param camera  the new camera
	 */
	public void setCamera(RotatePoint camera){this.camera = newcamera=camera;}

	/**
	 * Clear all at.
	 */
	public void clearAllAt()
	{
		mode_node=false;
		atRoom=null;
		atBuilding=null;
		atFloor=null;
		atNode=null;
		atRoute=null;
	}

	/**
	 * Fix bad angle.
	 *
	 * @param c the c
	 * @param n the n
	 */
	private void fixBadAngle(RotatePoint c, RotatePoint n)
	{
		if(c.rotate_z-n.rotate_z>180f)n.rotate_z+=360;
		else if(c.rotate_z-n.rotate_z<-180f)n.rotate_z-=360;
	}

	/**
	 * The set2dmode.
	 */
	public void toggle2DMode()
	{
		if(!(trans2DCount>0&&trans2DCount<TRANS_LENGTH-1))
		{
			if(mode_draw2D) trans2DCount=TRANS_LENGTH-1;
			else trans2DCount=1;
		}
		fixBadAngle(camera,mode2d_pos);
		mode_draw2D=!mode_draw2D;
	}

	private Node getClosestNode()
	{
		int i_min=0, dist=0;
		for(int i=0;i<nodes.length;i++)
		{
			int min=nodes[i].distanceTo(camera);
			if(min<dist||i==0)
			{
				i_min=i;
				dist=min;
			}
		}
		if(IndMap.MOVE_DEBUG)IndMap.log(IndMap.DEBUG, "MOVE*", "Closest Node: "+nodes[i_min].toString());
		return nodes[i_min];
	}

	/**
	 * Toggle node mode.
	 */
	public void toggleNodeMode()
	{
		if(!mode_node)
		{
			Node closest=getClosestNode();
			if(atRoute!=null&&!atRoute.isMember(closest))setAtRoute(null);
			setAtNode(closest);
		}
		mode_node=!mode_node;
		if(IndMap.MOVE_DEBUG)IndMap.log(IndMap.DEBUG, "MOVE*", "NodeMode: "+mode_node);
	}

	/**
	 * Sets the at building.
	 * @param atBuilding  the new at building
	 */
	public void setAtBuilding(Building atBuilding)
	{
		this.atBuilding = atBuilding;
		if(atBuilding!=null)
		{
			Building b=atBuilding;
			float le=b.getLength(), wi=b.getWidth();
			float z=0f;
			if(atNode!=null)z=atNode.getPoint().z;
			else z=b.getPosition().z;
			Point r=new Point(
					b.getPosition().x+le/2,
					b.getPosition().y+wi/2,
					z-(le>wi?le:wi)*0.5f/Math.tan(IndMapRenderer.PROJ_ANGLE/2)   //TODO Midlertidig. Regne ut ndvendig avstand gitt perspektivvinkel osv
			);
			mode2d_pos=new RotatePoint(r,0.0f,-90f,b.getXyRotate()-(le>wi?90f:0f));
			if(IndMap.MOVE_DEBUG)IndMap.log(IndMap.DEBUG, "MOVE", "In Building: "+atBuilding.toString()+" 2D mode pos: "+mode2d_pos.toString());
		}
		else
		{
			Point p=camera;
			mode2d_pos=new RotatePoint(p.x, p.y, 2000f, 0f, -90f, 90f);
		}
	}

	/**
	 * Sets the at floor.
	 * @param atFloor  the new at floor
	 */
	public void setAtFloor(Floor atFloor){this.atFloor = atFloor;}

	/**
	 * Sets the at room.
	 * @param atRoom  the new at room
	 */
	public void setAtRoom(Room atRoom){this.atRoom = atRoom;}

	/**
	 * Sets the at room.
	 *
	 * @param id the new at room
	 */
	public void setAtRoom(int id)
	{
		Room r=null;
		for(Building b : buildings)
		{
			r=(Room) Room.searchById(b.getRooms(), id);
			if(r!=null)break;
		}
		setAtRoom(r);
	}

	/**
	 * Sets the at route.
	 * @param atRoute  the new at route
	 */
	public void setAtRoute(Route atRoute)
	{
		if(IndMap.MOVE_DEBUG)IndMap.log(IndMap.DEBUG, "MOVE*", "Setting Route! "+(atRoute!=null?atRoute.toString():"null"));
		try{
			this.atRoute = atRoute;
			if(atRoute!=null)
			{
				if(!atRoute.gotoThis(atNode))
				{
					setAtNode(atRoute.gotoStart());
				}
			}
		}catch (Exception e) {
			IndMap.log(IndMap.ERROR, TAG, "setAtRoute "+e.toString());
		}
	}

	protected Node getNextNode()
	{
		if(nextNodeIndex<0)return atNode;
		if(atNode==null)return null;

		return atNode.getNeighbor(nextNodeIndex);
	}

	private void rotateToNextNode(boolean toRight)
	{
		if(atNode==null)return;
		if(atNode.numberOfNeighbors()<1)return;
		if(toRight)
		{
			nextNodeIndex++;
			if(nextNodeIndex>=atNode.numberOfNeighbors())nextNodeIndex=0;
		}
		else
		{
			nextNodeIndex--;
			if(nextNodeIndex<0)nextNodeIndex=atNode.numberOfNeighbors()-1;
		}
		Node next=atNode.getNeighbor(nextNodeIndex);
		Route dum=Route.makeDummy(atNode, next);

		float s=dum.getSiderot(),t=dum.getUprot();

		if(Float.isNaN(s))s=camera.rotate_z;
		if(Float.isNaN(t))t=camera.rotate_y;

		if(IndMap.MOVE_DEBUG)IndMap.log(IndMap.DEBUG, "MOVE", "SideRot: "+s);
		if(IndMap.MOVE_DEBUG)IndMap.log(IndMap.DEBUG, "MOVE", "UpRot: "+t);

		if(toRight){if(s<camera.rotate_z)s+=360f;}
		else if(s>camera.rotate_z)s-=360f;
		moveTo(new RotatePoint(transGoal, 0.0f, t, s), TRANS_ALL);
	}

	private int getBestNeighborIndex(Node n)
	{
		int best=0;
		float bestang=Float.POSITIVE_INFINITY;
		if(IndMap.MOVE_DEBUG)IndMap.log(IndMap.DEBUG, "MOVE*", "*GETTING best neighbor for: "+n.toString()+" Current angle: "+camera.rotate_z);
		for(int i=0;i<n.numberOfNeighbors();i++)
		{
			Node ne=n.getNeighbor(i);
			Point a=n.getPoint();
			Point b=ne.getPoint();
			float ang;
			if(a.x==b.x&&a.y==b.y)ang=Math.abs(720f+camera.rotate_y-n.getNeighborAngle(i));
			else ang=Math.abs(camera.rotate_z-n.getNeighborAngle(i));

			if(IndMap.MOVE_DEBUG)IndMap.log(IndMap.DEBUG, "MOVE*", "  - Found: "+ne.toString());
			if(ang<bestang)
			{
				bestang=ang;
				best=i;
			}
			if(IndMap.MOVE_DEBUG)IndMap.log(IndMap.DEBUG, "MOVE*", "     - Angle comb1: "+ang);
			ang=Math.abs(camera.rotate_z+360f-n.getNeighborAngle(i));
			if(ang<bestang)
			{
				bestang=ang;
				best=i;
			}
			if(IndMap.MOVE_DEBUG)IndMap.log(IndMap.DEBUG, "MOVE*", "     - Angle comb2: "+ang);
			ang=Math.abs(camera.rotate_z-360f-n.getNeighborAngle(i));
			if(ang<bestang)
			{
				bestang=ang;
				best=i;
			}			
			if(IndMap.MOVE_DEBUG)IndMap.log(IndMap.DEBUG, "MOVE*", "     - Angle comb3: "+ang);
		}
		if(IndMap.MOVE_DEBUG)IndMap.log(IndMap.DEBUG, "MOVE*", "  ** BEST ANGLE: : "+bestang);

		return best;
	}

	/**
	 * Sets the at node.
	 * @param to  the new at node
	 */
	public void setAtNode(Node to)
	{
		try{
			if(to!=null)
			{
				if(IndMap.MOVE_DEBUG)IndMap.log(IndMap.DEBUG, "MOVE*", "Going to node: "+to.toString());
				camera=newcamera;
				transCount=0;

				if(atRoute!=null)
				{
					mode_node=true;
					float r=atRoute.getSiderot(),t=atRoute.getUprot();
					if(Float.isNaN(r))r=camera.rotate_z;
					if(Float.isNaN(t))t=camera.rotate_y;
					moveTo(new RotatePoint(to.getPoint(), 0.0f, t, r), TRANS_ALL);
				}
				else
				{
					Node next;
					if(to.numberOfNeighbors()>0)
					{
						nextNodeIndex=getBestNeighborIndex(to);
						next=to.getNeighbor(nextNodeIndex);
					}
					else
					{
						nextNodeIndex=-1;
						if(atNode==null)next=new Node(null, null, null, null, newcamera);
						else next=atNode;
					}

					Route dum=Route.makeDummy(to, next);
					float r=dum.getSiderot(),t=dum.getUprot();
					if(Float.isNaN(r))r=newcamera.rotate_z;
					if(Float.isNaN(t))t=newcamera.rotate_y;
					
					if(to.numberOfNeighbors()==1)t=0.0f;
					
					moveTo(new RotatePoint(to.getPoint(), 0.0f, t, r), (to.numberOfNeighbors()==1?TRANS_NOT_ROT:TRANS_ALL));
				}

				Room r=to.getRoom();
				if(IndMap.MOVE_DEBUG)IndMap.log(IndMap.DEBUG, "MOVE*", "  - Has room? "+(r!=null));
				if(r!=null)
				{
					setAtRoom(r);
					setAtFloor(r.getFloor());
					setAtBuilding(r.getBuilding());
				}
				else
				{
					Floor f=to.getFloor();
					if(f!=null)
					{
						setAtRoom(null);
						setAtFloor(f);
						setAtBuilding(f.getBuilding());
					}
				}
			}
			else mode_node=false;
			last   = atNode;
			atNode = to;
		}catch (Exception e) {
			IndMap.log(IndMap.ERROR, TAG, "setAtNode "+e.toString());
		}
	}
	/**
	 * Sets the at node.
	 *
	 * @param id the new at node
	 */
	public void setAtNode(int id){setAtNode((Node) Node.searchById(nodes, id));}

	/**
	 * Gets the at building.
	 * @return  the at building
	 */
	public Building getAtBuilding(){return atBuilding;}

	/**
	 * Checks if is at.
	 *
	 * @param b the b
	 * @return true, if is at
	 */
	public boolean isAt(Building b)
	{
		if(atBuilding==null) return false;
		return atBuilding.equals(b);
	}

	/**
	 * Gets the at floor.
	 * @return  the at floor
	 */
	public Floor getAtFloor(){return atFloor;}

	/**
	 * Checks if is at.
	 *
	 * @param f the f
	 * @return true, if is at
	 */
	public boolean isAt(Floor f)
	{
		if(atFloor==null)return false;
		return atFloor.equals(f);
	}

	/**
	 * Gets the at room.
	 * @return  the at room
	 */
	public Room getAtRoom(){return atRoom;}

	/**
	 * Checks if is at.
	 *
	 * @param r the r
	 * @return true, if is at
	 */
	public boolean isAt(Room r)
	{
		if(atRoom==null)return false;
		return atRoom.equals(r);
	}

	/**
	 * Gets the at route.
	 * @return  the at route
	 */
	public Route getAtRoute(){return atRoute;}

	/**
	 * Checks if is at.
	 *
	 * @param r the r
	 * @return true, if is at
	 */
	public boolean isAt(Route r){return atRoute.equals(r);}

	/**
	 * Gets the at node.
	 * @return  the at node
	 */
	public Node getAtNode(){return atNode;}

	/**
	 * Checks if is at.
	 *
	 * @param n the n
	 * @return true, if is at
	 */
	public boolean isAt(Node n){return atNode.equals(n);}

	/**
	 * Gets the buildings.
	 * @return  the buildings
	 */
	public Building[] getBuildings(){return buildings;}

	/**
	 * LFEThread.
	 *
	 * @param statement the statement
	 * @param context the context
	 * @param h the h
	 * @return the lFE thread
	 */
	public LFEThread getLFEThread(MysqlViaPHP statement,Context context,Handler h){return new LFEThread(statement, context, h);}

	/**
	 * Draw.
	 *
	 * @param gl the gl
	 */
	public void draw(GL10 gl)
	{
		if(loadState)return;
		RotatePoint o = camera, q=transGoal,r=mode2d_pos;
		gl.glScalef(0.1f, 0.1f, 0.1f);
		gl.glRotatef(o.rotate_x, -1.0f, 0.0f, 0.0f); //Tilt to side

		RotatePoint p=newcamera=new RotatePoint(
				o.x        * (float)(TRANS_LENGTH-transCount)/TRANS_LENGTH + q.x        * transCount/TRANS_LENGTH, 
				o.y        * (float)(TRANS_LENGTH-transCount)/TRANS_LENGTH + q.y        * transCount/TRANS_LENGTH, 
				o.z        * (float)(TRANS_LENGTH-transCount)/TRANS_LENGTH + q.z        * transCount/TRANS_LENGTH, 
				0.0f, 
				o.rotate_y * (float)(TRANS_LENGTH-transCount)/TRANS_LENGTH + q.rotate_y * transCount/TRANS_LENGTH, 
				o.rotate_z * (float)(TRANS_LENGTH-transCount)/TRANS_LENGTH + q.rotate_z * transCount/TRANS_LENGTH
		);

		//Up rotate
		gl.glRotatef(
				(trans2DCount>0?(float)(TRANS_LENGTH-trans2DCount)/TRANS_LENGTH:1.0f) * p.rotate_y+
				(trans2DCount>0?r.rotate_y*trans2DCount/TRANS_LENGTH:0),		// 2D Mode uprotate				
				-1.0f, 0.0f, 0.0f);

		// Sideways rotate
		gl.glRotatef(
				(trans2DCount>0?(float)(TRANS_LENGTH-trans2DCount)/TRANS_LENGTH:1.0f) * p.rotate_z+
				(trans2DCount>0?r.rotate_z*trans2DCount/TRANS_LENGTH:0), 		// 2D Mode siderotate
				0.0f, 1.0f, 0.0f);

		//Position
		gl.glTranslatef(
				(trans2DCount>0?(float)(TRANS_LENGTH-trans2DCount)/TRANS_LENGTH:1.0f) * -p.y -
				(trans2DCount>0?(float)r.y*trans2DCount/TRANS_LENGTH:0.0f),		// 2D Mode y-pos

				(trans2DCount>0?(float)(TRANS_LENGTH-trans2DCount)/TRANS_LENGTH:1.0f) * -p.z-World.FROM_FLOOR -
				(trans2DCount>0?(float)r.z*trans2DCount/TRANS_LENGTH:0.0f),		// 2D Mode z-pos

				(trans2DCount>0?(float)(TRANS_LENGTH-trans2DCount)/TRANS_LENGTH:1.0f) * p.x +
				(trans2DCount>0?(float)r.x*trans2DCount/TRANS_LENGTH:0.0f)		// 2D Mode x-pos
		);

		if(trans2DCount>TRANS_LENGTH/4)
		{
			gl.glPushMatrix();
			gl.glTranslatef(p.y,p.z+FROM_FLOOR,-p.x);
			gl.glRotatef(p.rotate_z+180f, 0.0f, -1.0f, 0.0f);
			gl.glScalef(4f, 4f, 4f);

			Colour c=COL_MODE_2D_AT_POINT;
			int co=Color.argb(255*trans2DCount/TRANS_LENGTH, c.red, c.green, c.blue);
			mode2Dpoint.position(0);
			gl.glColor4f (Color.red(co)/(float) 0xFF, Color.green(co)/(float) 0xFF, Color.blue(co)/(float) 0xFF, Color.alpha(co)/(float) 0xFF);
			gl.glEnableClientState(GL10.GL_VERTEX_ARRAY);
			gl.glVertexPointer(3, GL10.GL_FIXED, 0, mode2Dpoint);
			gl.glDrawArrays(GL10.GL_TRIANGLE_STRIP, 0, 5);
			gl.glDisableClientState(GL10.GL_VERTEX_ARRAY);

			gl.glPopMatrix();
		}
		if(transCount>=TRANS_LENGTH)
		{
			if(IndMap.MOVE_DEBUG)IndMap.log(IndMap.DEBUG, "MOVE", "Transform finished!");
			camera=q;
			transCount=0;
		}
		if(transCount>0&&transCount<TRANS_LENGTH)transCount++;
		else
		{
			if(camera.rotate_z>180.0f)camera.rotate_z-=360;
			if(camera.rotate_z<-180.0f)camera.rotate_z+=360;
		}

		if(trans2DCount>0&&trans2DCount<TRANS_LENGTH)
		{
			if(mode_draw2D)trans2DCount++;
			else trans2DCount--;
		}
		for(Building b : buildings)
		{
			b.draw(gl, this);
		}
		for(Node n : nodes)
		{
			n.draw(gl, this);
		}
		if(atRoute!=null)atRoute.draw(gl, this);
	}

	/**
	 * Load from internal.
	 *
	 * @param app the app
	 * @return true, if successful
	 */
	public boolean loadFromInternal(IndMap app)
	{
		loadState=true;
		ArrayList<Building> b=new ArrayList<Building>();
		ArrayList<Room> r = new ArrayList<Room>(100);		

		try{
			//Buildings
			Cursor c=app.getContentResolver().query(
					Uri.withAppendedPath(Provider.CONTENT_URI, Building.TABLE_NAME),
					null, null, null, null);
			if(c.moveToFirst())
			{
				do
				{
					Building bu=new Building(c);
					b.add(bu);
					r.addAll(bu.loadFromInternal(app));
				}while(c.moveToNext());
			}
			else //empty
			{
				c.close();
				app.loadFromExternalDialog("Internal storage empty. Need to load from external. This requires WLAN connection.");
				return false;
			}
			c.close();
			buildings=new Building[b.size()];
			b.toArray(buildings);

			ArrayList<Ap> a=new ArrayList<Ap>(50);
			ArrayList<Node> n=new ArrayList<Node>(50);

			//Load Nodes
			c=app.getContentResolver().query(
					Uri.withAppendedPath(Provider.CONTENT_URI, Node.TABLE_NAME),
					null, null, null, null);
			if(c.moveToFirst()){do {
				Node no=new Node(c);
				n.add(no);
			}while(c.moveToNext());}
			c.close();
			nodes=new Node[n.size()];
			n.toArray(nodes);

			Iterator<Room> i=r.iterator();
			while(i.hasNext())
			{
				Room parent=i.next();
				c=app.getContentResolver().query(Uri.withAppendedPath(Provider.CONTENT_URI, Node.TABLE_NAME+Room.TABLE_NAME+"/"+parent.getI()), null,null,null,null);
				if(c.moveToFirst())
				{
					try{
						do
						{
							Node child=(Node) Node.searchById(nodes, c.getInt(c.getColumnIndex(Node._ID)));
							if(child!=null)
							{
								child.setRoom(parent);
								if(IndMap.LOAD_INT_DEBUG)IndMap.log(IndMap.DEBUG, TAG, "Found Node: ["+child.toString()+"] for Room ["+parent.toString()+"]");
							}
						}while(c.moveToNext());
					}catch (Exception e) {
						IndMap.log(IndMap.ERROR, TAG, "Load internal Room-Node "+e.toString());
					}
				}
				c.close();
			}

			for(Building by:buildings)for(Floor f:by.getFloors())
			{
				c=app.getContentResolver().query(Uri.withAppendedPath(Provider.CONTENT_URI, Node.TABLE_NAME+Floor.TABLE_NAME+"/"+f.getI()), null,null,null,null);
				if(c.moveToFirst())
				{
					try{
						do
						{
							Node child=(Node) Node.searchById(nodes, c.getInt(c.getColumnIndex(Node._ID)));
							if(child!=null)
							{
								child.setFloor(f);
								if(IndMap.LOAD_INT_DEBUG)IndMap.log(IndMap.DEBUG, TAG, "Found Node: ["+child.toString()+"] for Floor ["+f.toString()+"]");
							}
						}while(c.moveToNext());
					}catch (Exception e) {
						IndMap.log(IndMap.ERROR, TAG, "Load internal Floor-Node "+e.toString());
					}
				}
				c.close();
			}

			//Load Ap's
			c=app.getContentResolver().query(Uri.withAppendedPath(Provider.CONTENT_URI, Ap.TABLE_NAME+"/bybssid"), null,null,null,null);
			if(c.moveToFirst())	do {a.add(new Ap(c)); }while(c.moveToNext());
			c.close();
			knownAPs=new Ap[a.size()];
			a.toArray(knownAPs);

			//Link APs and Nodes
			Iterator<Node> j=n.iterator();
			ArrayList<Integer> level;
			ArrayList<Ap> ap;
			if(IndMap.LOAD_INT_DEBUG)IndMap.log(IndMap.DEBUG, TAG, "Load Aps to node");
			while(j.hasNext())
			{
				Node parent=j.next();

				c=app.getContentResolver().query(Uri.withAppendedPath(Provider.CONTENT_URI, Ap.TABLE_NAME+Node.TABLE_NAME+"/"+parent.getI()), null,null,null,null);
				level= new ArrayList<Integer>();
				ap  = new ArrayList<Ap>();
				if(IndMap.LOAD_INT_DEBUG)IndMap.log(IndMap.DEBUG, TAG, "  -Node: "+parent.toString()+" got "+c.getCount()+" aps");
				if(c.moveToFirst())
				{
					do
					{
						int id=c.getInt(c.getColumnIndex(Ap._ID));
						int lev=c.getInt(c.getColumnIndex(Ap.LEVELatNODE));
						Ap child=(Ap) Ap.getApByBSSID(knownAPs, Ap.stringSSID2long(c.getString(c.getColumnIndex(Ap.BSSID)),Ap.BSSID_SEPERATOR));
						if(IndMap.LOAD_INT_DEBUG)IndMap.log(IndMap.DEBUG, TAG, "    -Ap: id: "+id+" lev: "+lev+" : "+(child!=null?child.toString():"null"));
						if(child!=null)
						{
							ap.add(child);
							level.add(new Integer(lev));
							child.addNode(parent);
						}
					}while(c.moveToNext());
					int[] levels=new int[level.size()];
					for(int h=0;h<level.size();h++)levels[h]=level.get(h);
					Ap[] aps=new Ap[ap.size()];
					ap.toArray(aps);
					parent.setAPsandLevels(aps,levels);
				}
				c.close();

				//Linking Nodes
				c=app.getContentResolver().query(Uri.withAppendedPath(Provider.CONTENT_URI, Node.TABLE_NAME+Node.TABLE_NAME+"/"+parent.getI()), null,null,null,null);
				if(IndMap.LOAD_INT_DEBUG)IndMap.log(IndMap.DEBUG, TAG, "  -Node: "+parent.toString()+" got "+c.getCount()+" neighbors");
				if(c.moveToFirst())
				{
					do
					{
						Node child=(Node) Node.searchById(nodes, c.getInt(c.getColumnIndex(Node._ID)));
						if(IndMap.LOAD_INT_DEBUG)IndMap.log(IndMap.DEBUG, TAG, "    -Found Node: "+(child!=null?child.toString():"null"));
						if(child!=null)parent.addNeighbor(child, c.getFloat(c.getColumnIndex(Node.CHILD_COST)));
					}while(c.moveToNext());
				}
				c.close();
				parent.generateBuffers();
				parent.sortNeighborsCalcAngles();
			}

			loadState=false;
		}catch (Exception e) {
			IndMap.log(IndMap.ERROR, TAG, "Load internal "+e.toString());
		}
		return true;
	}

	private void alertPlaceChange(Context c)
	{
		if(atNode!=null)
		{
			Floor f=atNode.getFloor();
			Room r=atNode.getRoom();
			if( f!=null && (
					(last!=null&&!f.equals(last.getFloor())) || 
					last==null
			))
				Toast.makeText(c, "At Floor: "+f.toString(), Toast.LENGTH_SHORT).show();

			else if( r!=null && (
					(last!=null&&!r.equals(last.getRoom())) || 
					last==null
			))
				Toast.makeText(c, "At Room: "+r.toString(), Toast.LENGTH_SHORT).show();
		}
	}
	/**
	 * Move.
	 *
	 * @param move the move
	 */
	public void move(int move,Context c)
	{
		switch (move)
		{
			case MOVE_UPD:
				setAtNode(atNode);
			case MOVE_FWD:
				if(mode_node)
				{
					if(IndMap.MOVE_DEBUG)IndMap.log(IndMap.DEBUG, "MOVE", ">>>>>>>>  FWD! <<<<<<<<<<");
					if(atRoute!=null)setAtNode(atRoute.gotoNext());
					else setAtNode(getNextNode());

					alertPlaceChange(c);
				}
				else
				{
					camera.x+=scale*STEP_FWD*Math.cos(Math.toRadians(camera.rotate_z));
					camera.y+=scale*STEP_FWD*Math.sin(Math.toRadians(camera.rotate_z));
				}
				break;
			case MOVE_BWD:
				if(mode_node)
				{
					if(IndMap.MOVE_DEBUG)IndMap.log(IndMap.DEBUG, "MOVE", ">>>>>>>>  BWD! <<<<<<<<<<");
					if(atRoute!=null)setAtNode(atRoute.gotoPrev());
					else setAtNode(last);
					alertPlaceChange(c);
				}
				else
				{
					camera.x-=scale*STEP_FWD*Math.cos(Math.toRadians(camera.rotate_z));
					camera.y-=scale*STEP_FWD*Math.sin(Math.toRadians(camera.rotate_z));
				}
				break;
			case MOVE_LEFT:
				if(mode_node)
				{
					if(IndMap.MOVE_DEBUG)IndMap.log(IndMap.DEBUG, "MOVE", ">>>>>>>>  LEFT! <<<<<<<<<<");
					rotateToNextNode(false);
				}
				else
				{
					camera.x+=scale*STEP_SIDE*Math.sin(Math.toRadians(camera.rotate_z));
					camera.y-=scale*STEP_SIDE*Math.cos(Math.toRadians(camera.rotate_z));
				}
				break;
			case MOVE_RIGHT:
				if(mode_node)
				{
					if(IndMap.MOVE_DEBUG)IndMap.log(IndMap.DEBUG, "MOVE", ">>>>>>>>  RIGHT! <<<<<<<<<<");
					rotateToNextNode(true);
				}
				else
				{
					camera.x-=scale*STEP_SIDE*Math.sin(Math.toRadians(camera.rotate_z));
					camera.y+=scale*STEP_SIDE*Math.cos(Math.toRadians(camera.rotate_z));
				}				
				break;

			case MOVE_ROT_UP:camera.rotate_y+=STEP_UP_ROT;break;
			case MOVE_ROT_DOWN:camera.rotate_y-=STEP_UP_ROT;break;
			case MOVE_ROT_LEFT:camera.rotate_z-=STEP_SIDE_ROT;break;
			case MOVE_ROT_RIGHT:camera.rotate_z+=STEP_SIDE_ROT;break;

			case MOVE_UP: if(!mode_node)camera.z+=STEP_SIDE;break;
			case MOVE_DOWN: if(!mode_node)camera.z-=STEP_SIDE;break;

			default:
				break;
		}
	}

	/**
	 * Move to.
	 *
	 * @param newPos the new pos
	 * @param transMode the trans mode
	 */
	public void moveTo(RotatePoint newPos, final short transMode)
	{
		try{
			camera=newcamera;
			switch (transMode)
			{
				case TRANS_NOT_MOVE:
					//					fixBadAngle(camera,newPos);
					transGoal=new RotatePoint(newcamera, newPos);
					break;
				case TRANS_NOT_ROT:
					transGoal=new RotatePoint(newPos,newcamera);
					break;

				default:
					//				RotatePoint p=camera;
					//				if(p.rotate_z-newPos.rotate_z>180f)newPos.rotate_z+=360;
					//				else if(p.rotate_z-newPos.rotate_z<-180f)newPos.rotate_z-=360;
					fixBadAngle(newcamera,newPos);
					transGoal=newPos;
			}
			transCount=1;
		}catch (Exception e) {
			IndMap.log(IndMap.ERROR, TAG, "moveTo "+e.toString());
		}
	}

	/**
	 * Export.
	 *
	 * @param app the app
	 */
	public void export(Context app)
	{
		final String EXPORT_FILE_NAME = "export.sql";
		try{
			File mfile=new File(app.getExternalFilesDir(null),EXPORT_FILE_NAME);
			mfile.createNewFile();

			FileOutputStream fOut =  new FileOutputStream(mfile);
			BufferedOutputStream bos = new BufferedOutputStream( fOut );

			Cursor c;
			StringBuilder sb=new StringBuilder("--EXPORT OF Ap, ApNode, Node and NodeNode SQL from IndMap \n\n");
			QueryBuilder qb;

			c=app.getContentResolver().query(Uri.withAppendedPath(Provider.CONTENT_URI, Ap.TABLE_NAME), null,null,null,null);


			if(c.moveToFirst())
			{
				sb.append("--"+Ap.TABLE_NAME+" table\n\n");
				qb=new QueryBuilder(QueryBuilder.INSERT);
				qb.setTable(Ap.TABLE_NAME);
				qb.addFields(Ap.Entety.PROJ_ALL);
				do
				{
					qb.newRow();
					qb.addValue(c.getInt(c.getColumnIndex(Ap._ID)));
					qb.addValue(c.getString(c.getColumnIndex(Ap.NAME)));
					qb.addValue(c.getString(c.getColumnIndex(Ap.BSSID)));
				}while(c.moveToNext());
				sb.append(qb.getSqlString()+"\n\n\n");
				bos.write(sb.toString().getBytes());
			}
			c.close();

			c=app.getContentResolver().query(Uri.withAppendedPath(Provider.CONTENT_URI, Ap.TABLE_NAME+Node.TABLE_NAME), null,null,null,null);
			if(c.moveToFirst())
			{
				sb=new StringBuilder("--"+Ap.TABLE_NAME+Node.TABLE_NAME+" table\n\n");
				qb=new QueryBuilder(QueryBuilder.INSERT);
				qb.setTable(Ap.TABLE_NAME+Node.TABLE_NAME);
				qb.addFields(new String[]{Ap.PARENT,Node.PARENT,Ap.LEVELatNODE});
				do
				{
					qb.newRow();
					qb.addValue(c.getInt(c.getColumnIndex(Ap.PARENT)));
					qb.addValue(c.getInt(c.getColumnIndex(Node.PARENT)));
					qb.addValue(c.getInt(c.getColumnIndex(Ap.LEVELatNODE)));
				}
				while(c.moveToNext());
				sb.append(qb.getSqlString()+"\n\n\n");
				bos.write(sb.toString().getBytes());
			}
			c.close();

			c=app.getContentResolver().query(
					Uri.withAppendedPath(Provider.CONTENT_URI, Node.TABLE_NAME),
					null, null, null, null);
			if(c.moveToFirst())
			{
				sb=new StringBuilder("--"+Node.TABLE_NAME+" table\n\n");
				qb=new QueryBuilder(QueryBuilder.INSERT);
				qb.setTable(Node.TABLE_NAME);
				qb.addFields(Node.Entety.PROJ_ALL);
				do
				{
					//{_ID,NAME,DESC,Room.PARENT,Floor.PARENT,Building.PARENT,X,Y,Z}
					qb.newRow();
					qb.addValue(c.getInt(c.getColumnIndex(Node._ID)));
					qb.addValue(c.getString(c.getColumnIndex(Node.NAME)));
					qb.addValue(c.getString(c.getColumnIndex(Node.DESC)));
					qb.addValue(c.getInt(c.getColumnIndex(Room.PARENT)));
					qb.addValue(c.getInt(c.getColumnIndex(Floor.PARENT)));
					qb.addValue(c.getInt(c.getColumnIndex(Building.PARENT)));
					qb.addValue(c.getFloat(c.getColumnIndex(Node.X)));
					qb.addValue(c.getFloat(c.getColumnIndex(Node.Y)));
					qb.addValue(c.getFloat(c.getColumnIndex(Node.Z)));
				}
				while(c.moveToNext());
				sb.append(qb.getSqlString()+"\n\n\n");
				bos.write(sb.toString().getBytes());
			}
			c.close();

			c=app.getContentResolver().query(
					Uri.withAppendedPath(Provider.CONTENT_URI, Node.TABLE_NAME+Node.TABLE_NAME), 
					null,null,null,null);
			if(c.moveToFirst())
			{
				sb=new StringBuilder("--"+Node.TABLE_NAME+Node.TABLE_NAME+" table\n\n");
				qb=new QueryBuilder(QueryBuilder.INSERT);
				qb.setTable(Node.TABLE_NAME+Node.TABLE_NAME);
				qb.addFields(new String[]{Node.PARENTA,Node.PARENTB,Node.CHILD_COST});
				do
				{
					qb.newRow();
					qb.addValue(c.getInt(c.getColumnIndex(Node.PARENTA)));
					qb.addValue(c.getInt(c.getColumnIndex(Node.PARENTB)));
					qb.addValue(c.getFloat(c.getColumnIndex(Node.CHILD_COST)));
				}
				while(c.moveToNext());
				sb.append(qb.getSqlString()+"\n\n\n");
				bos.write(sb.toString().getBytes());
			}
			c.close();

			bos.close();

		}
		catch (Exception e) {
			IndMap.log(IndMap.ERROR, TAG, "Export "+e.toString());
			Toast.makeText(app, e.toString(), Toast.LENGTH_LONG);
		}
	}
}